# sage_setup: distribution = sagemath-categories
"""
A parser for symbolic equations and expressions

It is both safer and more powerful than using Python's eval, as one has
complete control over what names are used (including dynamically creating
variables) and how integer and floating point literals are created.

AUTHOR:

- Robert Bradshaw 2008-04 (initial version)
"""
# ***************************************************************************
#       Copyright (C) 2008 Robert Bradshaw <robertwb@math.washington.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ***************************************************************************

from cpython.list cimport PyList_Append

import math


def foo(*args, **kwds):
    """
    This is a function for testing that simply returns the arguments and
    keywords passed into it.

    EXAMPLES::

        sage: from sage.misc.parser import foo
        sage: foo(1, 2, a=3)
        ((1, 2), {'a': 3})
    """
    return args, kwds


function_map = {
  'foo': foo,
  'sqrt': math.sqrt,
  'sin': math.sin,
  'cos': math.cos,
  'tan': math.tan,
}


cdef enum token_types:
    # leave room for ASCII character tokens such as '+'
    INT = 128
    FLOAT
    NAME
    EOS
    ERROR

    LESS_EQ
    GREATER_EQ
    NOT_EQ
    MATRIX

enum_map = {
  INT: 'INT',
  FLOAT: 'FLOAT',
  NAME: 'NAME',
  EOS: 'EOS',
  ERROR: 'ERROR',
  LESS_EQ: 'LESS_EQ',
  GREATER_EQ: 'GREATER_EQ',
  NOT_EQ: 'NOT_EQ',
  MATRIX: 'MATRIX',
}


def token_to_str(int token):
    """
    For speed reasons, tokens are integers. This function returns a string
    representation of a given token.

    EXAMPLES::

        sage: from sage.misc.parser import Tokenizer, token_to_str
        sage: t = Tokenizer("+ 2")
        sage: token_to_str(t.next())
        '+'
        sage: token_to_str(t.next())
        'INT'
    """
    try:
        return enum_map[token]
    except KeyError:
        return chr(token)


cdef inline bint is_alphanumeric(c) noexcept:
    return c.isalnum() or c == '_'

cdef inline bint is_whitespace(c) noexcept:
    return c.isspace()


cdef class Tokenizer:
    cdef str s
    cdef int token
    cdef int pos
    cdef int last_pos

    def __init__(self, s):
        r"""
        This class takes a string and turns it into a list of tokens for use
        by the parser.

        The tokenizer wraps a string object, to tokenize a different string
        create a new tokenizer.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer
            sage: Tokenizer("1.5+2*3^4-sin(x)").test()
            ['FLOAT(1.5)', '+', 'INT(2)', '*', 'INT(3)', '^', 'INT(4)', '-', 'NAME(sin)', '(', 'NAME(x)', ')']

        The single character tokens are given by::

            sage: Tokenizer("+-*/^(),=<>[]{}").test()
            ['+', '-', '*', '/', '^', '(', ')', ',', '=', '<', '>', '[', ']', '{', '}']

        Two-character comparisons accepted are::

            sage: Tokenizer("<= >= != == **").test()
            ['LESS_EQ', 'GREATER_EQ', 'NOT_EQ', '=', '^']

        Integers are strings of 0-9::

            sage: Tokenizer("1 123 9879834759873452908375013").test()
            ['INT(1)', 'INT(123)', 'INT(9879834759873452908375013)']

        Floating point numbers can contain a single decimal point and possibly exponential notation::

            sage: Tokenizer("1. .01 1e3 1.e-3").test()
            ['FLOAT(1.)', 'FLOAT(.01)', 'FLOAT(1e3)', 'FLOAT(1.e-3)']

        Note that negative signs are not attached to the token::

            sage: Tokenizer("-1 -1.2").test()
            ['-', 'INT(1)', '-', 'FLOAT(1.2)']

        Names are alphanumeric sequences not starting with a digit::

            sage: Tokenizer("a a1 _a_24").test()
            ['NAME(a)', 'NAME(a1)', 'NAME(_a_24)']

        There is special handling for matrices::

            sage: Tokenizer("matrix(a)").test()
            ['MATRIX', '(', 'NAME(a)', ')']

        Anything else is an error::

            sage: Tokenizer("&@~").test()
            ['ERROR', 'ERROR', 'ERROR']

        No attempt for correctness is made at this stage::

            sage: Tokenizer(") )( 5e5e5").test()
            [')', ')', '(', 'FLOAT(5e5)', 'NAME(e5)']
            sage: Tokenizer("?$%").test()
            ['ERROR', 'ERROR', 'ERROR']

        TESTS:

        Check support for unicode characters (:issue:`29280`)::

            sage: Tokenizer("λ+α_β0 Γ^ω").test()
            ['NAME(λ)', '+', 'NAME(α_β0)', 'NAME(Γ)', '^', 'NAME(ω)']
        """
        self.pos = 0
        self.last_pos = 0
        self.s = s

    def test(self):
        """
        This is a utility function for easy testing of the tokenizer.

        Destructively read off the tokens in self, returning a list of string
        representations of the tokens.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer
            sage: t = Tokenizer("a b 3")
            sage: t.test()
            ['NAME(a)', 'NAME(b)', 'INT(3)']
            sage: t.test()
            []
        """
        all = []
        cdef int token = self.next()
        while token != EOS:
            if token in [INT, FLOAT, NAME]:
                all.append("%s(%s)" % (token_to_str(token), self.last_token_string()))
            else:
                all.append(token_to_str(token))
            token = self.next()
        return all

    cpdef reset(self, int pos=0):
        """
        Reset the tokenizer to a given position.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer
            sage: t = Tokenizer("a+b*c")
            sage: t.test()
            ['NAME(a)', '+', 'NAME(b)', '*', 'NAME(c)']
            sage: t.test()
            []
            sage: t.reset()
            sage: t.test()
            ['NAME(a)', '+', 'NAME(b)', '*', 'NAME(c)']
            sage: t.reset(3)
            sage: t.test()
            ['*', 'NAME(c)']

        No care is taken to make sure we don't jump in the middle of a token::

            sage: t = Tokenizer("12345+a")
            sage: t.test()
            ['INT(12345)', '+', 'NAME(a)']
            sage: t.reset(2)
            sage: t.test()
            ['INT(345)', '+', 'NAME(a)']
        """
        self.pos = self.last_pos = pos

    cdef int find(self) except -1:
        """
        This function actually does all the work, and is extensively tested
        above.
        """
        cdef bint seen_exp, seen_decimal
        cdef int type
        cdef str s = self.s
        cdef int pos = self.pos
        cdef int s_len = len(s)

        # skip whitespace
        if pos < s_len and is_whitespace(s[pos]):
            while pos < s_len and is_whitespace(s[pos]):
                pos += 1
            self.pos = pos

        # end of string
        if pos == s_len:
            return EOS

        # diphthongs
        if pos+1 < s_len:
            if s[pos+1] == '=':
                if s[pos] == '<':
                    self.pos += 2
                    return LESS_EQ
                elif s[pos] == '>':
                    self.pos += 2
                    return GREATER_EQ
                elif s[pos] == '!':
                    self.pos += 2
                    return NOT_EQ
                elif s[pos] == '=':
                    self.pos += 2
                    return ord('=')

            elif s[pos] == '*' and s[pos+1] == '*':
                self.pos += 2
                return ord('^')

        # simple tokens
        if s[pos] in "+-*/^()=><,[]{}!":
            type = ord(s[pos])
            self.pos += 1
            return type

        # numeric literals
        if s[pos].isdigit() or s[pos] == '.':
            type = INT
            seen_exp = False
            seen_decimal = False
            while pos < s_len:
                if s[pos].isdigit():
                    pass
                elif s[pos] == '.':
                    if seen_decimal or seen_exp:
                        self.pos = pos
                        return type
                    else:
                        type = FLOAT
                        seen_decimal = True
                elif s[pos] == 'e' or s[pos] == 'E':
                    if seen_exp:
                        self.pos = pos
                        return type
                    else:
                        type = FLOAT
                        seen_exp = True
                elif s[pos] == '+' or s[pos] == '-':
                    if not (seen_exp and (s[pos-1] == 'e' or s[pos-1] == 'E')):
                        self.pos = pos
                        return type
                else:
                    break
                pos += 1
            self.pos = pos
            return type

        # name literals
        if is_alphanumeric(s[pos]):
            while pos < s_len and is_alphanumeric(s[pos]):
                pos += 1
            # matrices
            if s[self.pos:pos] == 'matrix':
                self.pos = pos
                return MATRIX
            self.pos = pos
            return NAME

        pos += 1
        self.pos = pos
        return ERROR

    cpdef int next(self) noexcept:
        """
        Return the next token in the string.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer, token_to_str
            sage: t = Tokenizer("a+3")
            sage: token_to_str(t.next())
            'NAME'
            sage: token_to_str(t.next())
            '+'
            sage: token_to_str(t.next())
            'INT'
            sage: token_to_str(t.next())
            'EOS'
        """
        while self.pos < len(self.s) and is_whitespace(self.s[self.pos]):
            self.pos += 1
        self.last_pos = self.pos
        self.token = self.find()
        return self.token

    cpdef int last(self) noexcept:
        """
        Return the last token seen.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer, token_to_str
            sage: t = Tokenizer("3a")
            sage: token_to_str(t.next())
            'INT'
            sage: token_to_str(t.last())
            'INT'
            sage: token_to_str(t.next())
            'NAME'
            sage: token_to_str(t.last())
            'NAME'
        """
        return self.token

    cpdef int peek(self) noexcept:
        """
        Return the next token that will be encountered, without changing
        the state of ``self``.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer, token_to_str
            sage: t = Tokenizer("a+b")
            sage: token_to_str(t.peek())
            'NAME'
            sage: token_to_str(t.next())
            'NAME'
            sage: token_to_str(t.peek())
            '+'
            sage: token_to_str(t.peek())
            '+'
            sage: token_to_str(t.next())
            '+'
        """
        cdef int save_pos = self.pos
        cdef int token = self.find()
        self.pos = save_pos
        return token

    cpdef bint backtrack(self) except -2:
        """
        Put ``self`` in such a state that the subsequent call to ``next()``
        will return the same as if ``next()`` had not been called.

        Currently, one can only backtrack once.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer, token_to_str
            sage: t = Tokenizer("a+b")
            sage: token_to_str(t.next())
            'NAME'
            sage: token_to_str(t.next())
            '+'
            sage: t.backtrack()   # the return type is bint for performance reasons
            False
            sage: token_to_str(t.next())
            '+'
        """
        if self.pos == self.last_pos and self.token != EOS:
            raise NotImplementedError("Can only backtrack once.")
        else:
            self.pos = self.last_pos
            self.token = 0

    cpdef last_token_string(self):
        """
        Return the actual contents of the last token.

        EXAMPLES::

            sage: from sage.misc.parser import Tokenizer, token_to_str
            sage: t = Tokenizer("a - 1e5")
            sage: token_to_str(t.next())
            'NAME'
            sage: t.last_token_string()
            'a'
            sage: token_to_str(t.next())
            '-'
            sage: token_to_str(t.next())
            'FLOAT'
            sage: t.last_token_string()
            '1e5'
        """
        return self.s[self.last_pos:self.pos]


cdef class Parser:

    cdef integer_constructor
    cdef float_constructor
    cdef variable_constructor
    cdef callable_constructor
    cdef bint implicit_multiplication

    def __init__(self, make_int=int, make_float=float, make_var=str, make_function={}, bint implicit_multiplication=True):
        """
        Create a symbolic expression parser.

        INPUT:

        - ``make_int`` -- callable object to construct integers from strings (default: int)
        - ``make_float`` -- callable object to construct real numbers from strings (default: float)
        - ``make_var`` -- callable object to construct variables from strings (default: str)
          this may also be a dictionary of variable names
        - ``make_function`` -- callable object to construct callable functions from strings
          this may also be a dictionary
        - ``implicit_multiplication`` -- whether or not to accept implicit multiplication

        OUTPUT:

        The evaluated expression tree given by the string, where the above
        functions are used to create the leaves of this tree.

        EXAMPLES::

            sage: from sage.misc.parser import Parser
            sage: p = Parser()
            sage: p.parse("1+2")
            3
            sage: p.parse("1+2 == 3")
            True

            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.parse("a*b^c - 3a")                                                 # needs sage.symbolic
            a*b^c - 3*a

            sage: R.<x> = QQ[]
            sage: p = Parser(make_var={'x': x})
            sage: p.parse("(x+1)^5-x")
            x^5 + 5*x^4 + 10*x^3 + 10*x^2 + 4*x + 1
            sage: p.parse("(x+1)^5-x").parent() is R
            True

            sage: p = Parser(make_float=RR, make_var=var,                               # needs sage.symbolic
            ....:            make_function={'foo': (lambda x: x*x+x)})
            sage: p.parse("1.5 + foo(b)")                                               # needs sage.symbolic
            b^2 + b + 1.50000000000000
            sage: p.parse("1.9").parent()                                               # needs sage.symbolic
            Real Field with 53 bits of precision
        """
        self.integer_constructor = make_int
        self.float_constructor = make_float
        if not callable(make_var):
            make_var = LookupNameMaker(make_var)
        if not callable(make_function):
            make_function = LookupNameMaker(make_function)
        self.variable_constructor = make_var
        self.callable_constructor = make_function
        self.implicit_multiplication = implicit_multiplication

    def _variable_constructor(self):
        """
        Return the variable constructor of this parser.

        EXAMPLES::

            sage: from sage.calculus.calculus import SR_parser                          # needs sage.symbolic
            sage: SR_parser._variable_constructor()                                     # needs sage.symbolic
            <sage.misc.parser.LookupNameMaker...
        """
        return self.variable_constructor

    def _callable_constructor(self):
        """
        Return the callable constructor of this parser.

        EXAMPLES::

            sage: from sage.calculus.calculus import SR_parser                          # needs sage.symbolic
            sage: SR_parser._callable_constructor()                                     # needs sage.symbolic
            <sage.misc.parser.LookupNameMaker...
        """
        return self.callable_constructor

    cpdef parse(self, s, bint accept_eqn=True):
        """
        Parse the given string.

        EXAMPLES::

            sage: from sage.misc.parser import Parser
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.parse("E = m c^2")                                                  # needs sage.symbolic
            E == c^2*m
        """
        cdef Tokenizer tokens = Tokenizer(s)
        if tokens.peek() == MATRIX:
            tokens.next()
            expr = self.p_matrix(tokens)
        else:
            expr = self.p_eqn(tokens) if accept_eqn else self.p_expr(tokens)

        if tokens.next() != EOS:
            self.parse_error(tokens)
        return expr

    cpdef parse_expression(self, s):
        """
        Parse an expression.

        EXAMPLES::

            sage: from sage.misc.parser import Parser
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.parse_expression('a-3b^2')                                          # needs sage.symbolic
            -3*b^2 + a
        """
        cdef Tokenizer tokens = Tokenizer(s)
        expr = self.p_expr(tokens)
        if tokens.next() != EOS:
            self.parse_error(tokens)
        return expr

    cpdef parse_sequence(self, s):
        """
        Parse a (possibly nested) set of lists and tuples.

        EXAMPLES::

            sage: # needs sage.symbolic
            sage: from sage.misc.parser import Parser
            sage: p = Parser(make_var=var)
            sage: p.parse_sequence("1,2,3")
            [1, 2, 3]
            sage: p.parse_sequence("[1,2,(a,b,c+d)]")
            [1, 2, (a, b, c + d)]
            sage: p.parse_sequence("13")
            13
        """
        cdef Tokenizer tokens = Tokenizer(s)
        all = self.p_sequence(tokens)
        if tokens.next() != EOS:
            self.parse_error(tokens)
        if len(all) == 1 and isinstance(all, list):
            all = all[0]
        return all

    cpdef p_matrix(self, Tokenizer tokens):
        """
        Parse a matrix.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_matrix(Tokenizer("([a,0],[0,a])"))                                # needs sage.symbolic
            [a 0]
            [0 a]
        """
        cdef int token
        all = []
        if tokens.next() == c'(':
            token = c','
            while token == c',':
                all.append(self.p_list(tokens))
                token = tokens.next()

            if token == c')':
                from sage.matrix.constructor import matrix
                return matrix(all)
            else:
                self.parse_error(tokens, "Malformed matrix")
        else:
            self.parse_error(tokens, "Malformed matrix")

    cpdef p_sequence(self, Tokenizer tokens):
        """
        Parse a (possibly nested) set of lists and tuples.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_sequence(Tokenizer("[1+2,0]"))                                    # needs sage.symbolic
            [[3, 0]]
            sage: p.p_sequence(Tokenizer("(1,2,3) , [1+a, 2+b, (3+c), (4+d,)]"))        # needs sage.symbolic
            [(1, 2, 3), [a + 1, b + 2, c + 3, (d + 4,)]]
        """
        all = []
        cdef int token = c','
        while token == c',':
            token = tokens.peek()
            if token == MATRIX:
                tokens.next()
                obj = self.p_matrix(tokens)
            elif token == INT:
                # we optimize for this rather than going all the way to atom
                tokens.next()
                if tokens.peek() == c',':
                    obj = self.integer_constructor(tokens.last_token_string())
                else:
                    tokens.backtrack()
                    obj = self.p_eqn(tokens)
            elif token == c'[':
                obj = self.p_list(tokens)
            elif token == c'(':
                obj = self.p_tuple(tokens)
            elif token == EOS:
                return all
            elif token == c']' or token == c')':
                tokens.token = c','
                return all
            else:
                obj = self.p_eqn(tokens)
            PyList_Append(all, obj)
            token = tokens.next()

        tokens.backtrack()
        return all

    cpdef p_list(self, Tokenizer tokens):
        """
        Parse a list of items.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_list(Tokenizer("[1+2, 1e3]"))                                     # needs sage.symbolic
            [3, 1000.0]
            sage: p.p_list(Tokenizer("[]"))                                             # needs sage.symbolic
            []
        """
        cdef int token = tokens.next()
        if token != c'[':
            self.parse_error(tokens, "Malformed list")
        all = self.p_sequence(tokens)
        token = tokens.next()
        if token != c']':
            self.parse_error(tokens, "Malformed list")
        return all

    cpdef p_tuple(self, Tokenizer tokens):
        """
        Parse a tuple of items.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_tuple(Tokenizer("( (), (1), (1,), (1,2), (1,2,3), (1+2)^2, )"))   # needs sage.symbolic
            ((), 1, (1,), (1, 2), (1, 2, 3), 9)
        """
        cdef int start = tokens.pos
        cdef int token = tokens.next()
        cdef bint real_tuple = True
        if token != c'(':
            self.parse_error(tokens, "Malformed tuple")
        all = self.p_sequence(tokens)
        if len(all) == 1:
            if tokens.last() != c',':
                real_tuple = False
        token = tokens.next()
        if token != c')':
            self.parse_error(tokens, "Malformed tuple")
        if real_tuple:
            return tuple(all)
        else:
            token = tokens.peek()
            if token == c',' or token == EOS:
                return all[0]
            else:
                # we have to reparse the entire thing as an expression
                tokens.reset(start)
                return self.p_eqn(tokens)

# eqn ::= expr op expr | expr
    cpdef p_eqn(self, Tokenizer tokens):
        r"""
        Parse an equation or expression.

        This is the top-level node called by the \code{parse} function.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_eqn(Tokenizer("1+a"))                                             # needs sage.symbolic
            a + 1

            sage: # needs sage.symbolic
            sage: p.p_eqn(Tokenizer("a == b"))
            a == b
            sage: p.p_eqn(Tokenizer("a < b"))
            a < b
            sage: p.p_eqn(Tokenizer("a > b"))
            a > b
            sage: p.p_eqn(Tokenizer("a <= b"))
            a <= b
            sage: p.p_eqn(Tokenizer("a >= b"))
            a >= b
            sage: p.p_eqn(Tokenizer("a != b"))
            a != b
        """
        lhs = self.p_expr(tokens)
        cdef int op = tokens.next()
        if op == c'=':
            return lhs == self.p_expr(tokens)
        elif op == NOT_EQ:
            return lhs != self.p_expr(tokens)
        elif op == c'<':
            return lhs < self.p_expr(tokens)
        elif op == LESS_EQ:
            return lhs <= self.p_expr(tokens)
        elif op == c'>':
            return lhs > self.p_expr(tokens)
        elif op == GREATER_EQ:
            return lhs >= self.p_expr(tokens)
        else:
            tokens.backtrack()
            return lhs

# expr ::=  term | expr '+' term | expr '-' term
    cpdef p_expr(self, Tokenizer tokens):
        """
        Parse a list of one or more terms.

        EXAMPLES::

            sage: # needs sage.symbolic
            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)
            sage: p.p_expr(Tokenizer("a+b"))
            a + b
            sage: p.p_expr(Tokenizer("a"))
            a
            sage: p.p_expr(Tokenizer("a - b + 4*c - d^2"))
            -d^2 + a - b + 4*c
            sage: p.p_expr(Tokenizer("a - -3"))
            a + 3
            sage: p.p_expr(Tokenizer("a + 1 == b"))
            a + 1
        """
        # Note: this is left-recursive, so we can't just recurse
        cdef int op
        operand1 = self.p_term(tokens)
        op = tokens.next()
        while op == c'+' or op == c'-':
            operand2 = self.p_term(tokens)
            if op == c'+':
                operand1 = operand1 + operand2
            else:
                operand1 = operand1 - operand2
            op = tokens.next()
        tokens.backtrack()
        return operand1

# term ::=  factor | term '*' factor | term '/' factor
    cpdef p_term(self, Tokenizer tokens):
        """
        Parse a single term (consisting of one or more factors).

        EXAMPLES::

            sage: # needs sage.symbolic
            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)
            sage: p.p_term(Tokenizer("a*b"))
            a*b
            sage: p.p_term(Tokenizer("a * b / c * d"))
            a*b*d/c
            sage: p.p_term(Tokenizer("-a * b + c"))
            -a*b
            sage: p.p_term(Tokenizer("a*(b-c)^2"))
            a*(b - c)^2
            sage: p.p_term(Tokenizer("-3a"))
            -3*a
        """
        # Note: this is left-recursive, so we can't just recurse
        cdef int op
        operand1 = self.p_factor(tokens)
        op = tokens.next()
        if op == NAME and self.implicit_multiplication:
            op = c'*'
            tokens.backtrack()
        while op == c'*' or op == c'/':
            operand2 = self.p_factor(tokens)
            if op == c'*':
                operand1 = operand1 * operand2
            else:
                operand1 = operand1 / operand2
            op = tokens.next()
            if op == NAME and self.implicit_multiplication:
                op = c'*'
                tokens.backtrack()
        tokens.backtrack()
        return operand1

# factor ::=  '+' factor | '-' factor | power
    cpdef p_factor(self, Tokenizer tokens):
        """
        Parse a single factor, which consists of any number of unary +/-
        and a power.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: R.<t> = ZZ[['t']]
            sage: p = Parser(make_var={'t': t})
            sage: p.p_factor(Tokenizer("- -t"))
            t
            sage: p.p_factor(Tokenizer("- + - -t^2"))
            -t^2
            sage: p.p_factor(Tokenizer("t^11 * x"))
            t^11
        """
        cdef int token = tokens.next()
        if token == c'+':
            return self.p_factor(tokens)
        elif token == c'-':
            return -self.p_factor(tokens)
        else:
            tokens.backtrack()
            return self.p_power(tokens)

# power ::=  (atom | atom!) ^ factor | atom | atom!
    cpdef p_power(self, Tokenizer tokens):
        """
        Parses a power. Note that exponentiation groups right to left.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: R.<t> = ZZ[['t']]
            sage: p = Parser(make_var={'t': t})
            sage: p.p_factor(Tokenizer("-(1+t)^-1"))
            -1 + t - t^2 + t^3 - t^4 + t^5 - t^6 + t^7 - t^8 + t^9 - t^10 + t^11 - t^12 + t^13 - t^14 + t^15 - t^16 + t^17 - t^18 + t^19 + O(t^20)
            sage: p.p_factor(Tokenizer("t**2"))
            t^2
            sage: p.p_power(Tokenizer("2^3^2")) == 2^9
            True

            sage: # needs sage.symbolic
            sage: p = Parser(make_var=var)
            sage: p.p_factor(Tokenizer('x!'))
            factorial(x)
            sage: p.p_factor(Tokenizer('(x^2)!'))
            factorial(x^2)
            sage: p.p_factor(Tokenizer('x!^2'))
            factorial(x)^2
        """
        operand1 = self.p_atom(tokens)
        cdef int token = tokens.next()
        if token == c'^':
            operand2 = self.p_factor(tokens)
            return operand1 ** operand2
        elif token == c"!":
            from sage.functions.all import factorial
            operand1 = factorial(operand1)
            if tokens.peek() == c'^':
                tokens.next()
                operand2 = self.p_factor(tokens)
                return operand1 ** operand2
            else:
                return operand1
        else:
            tokens.backtrack()
            return operand1

# atom ::= int | float | name | '(' expr ')' | name '(' args ')'
    cpdef p_atom(self, Tokenizer tokens):
        """
        Parse an atom. This is either a parenthesized expression, a function call, or a literal name/int/float.

        EXAMPLES::

            sage: # needs sage.symbolic
            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var, make_function={'sin': sin})
            sage: p.p_atom(Tokenizer("1"))
            1
            sage: p.p_atom(Tokenizer("12"))
            12
            sage: p.p_atom(Tokenizer("12.5"))
            12.5
            sage: p.p_atom(Tokenizer("(1+a)"))
            a + 1
            sage: p.p_atom(Tokenizer("(1+a)^2"))
            a + 1
            sage: p.p_atom(Tokenizer("sin(1+a)"))
            sin(a + 1)
            sage: p = Parser(make_var=var,
            ....:            make_function={'foo': sage.misc.parser.foo})
            sage: p.p_atom(Tokenizer("foo(a, b, key=value)"))
            ((a, b), {'key': value})
            sage: p.p_atom(Tokenizer("foo()"))
            ((), {})
        """
        cdef int token = tokens.next()
        if token == INT:
            return self.integer_constructor(tokens.last_token_string())
        elif token == FLOAT:
            return self.float_constructor(tokens.last_token_string())
        elif token == NAME:
            name = tokens.last_token_string()
            token = tokens.next()
            if token == c'(':
                func = self.callable_constructor(name)
                args, kwds = self.p_args(tokens)
                token = tokens.next()
                if token != c')':
                    self.parse_error(tokens, "Bad function call")
                return func(*args, **kwds)
            else:
                tokens.backtrack()
                return self.variable_constructor(name)
        elif token == c'(':
            expr = self.p_expr(tokens)
            token = tokens.next()
            if token != c')':
                self.parse_error(tokens, "Mismatched parentheses")
            return expr
        else:
            self.parse_error(tokens)

# args = arg (',' arg)* | EMPTY
    cpdef p_args(self, Tokenizer tokens):
        """
        Return a ``list, dict`` pair.

        EXAMPLES::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser()
            sage: p.p_args(Tokenizer("1,2,a=3"))
            ([1, 2], {'a': 3})
            sage: p.p_args(Tokenizer("1, 2, a = 1+5^2"))
            ([1, 2], {'a': 26})
        """
        args = []
        kwds = {}
        if tokens.peek() == c')':
            return args, kwds
        cdef int token = c','
        while token == c',':
            arg = self.p_arg(tokens)
            if isinstance(arg, tuple):
                name, value = arg
                kwds[name] = value
            else:
                args.append(arg)
            token = tokens.next()
        tokens.backtrack()
        return args, kwds

# arg = expr | name '=' expr
    cpdef p_arg(self, Tokenizer tokens):
        """
        Return an ``expr``, or a ``(name, expr)`` tuple corresponding to a
        single function call argument.

        EXAMPLES:

        Parsing a normal expression::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_arg(Tokenizer("a+b"))                                             # needs sage.symbolic
            a + b

       A keyword expression argument::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_arg(Tokenizer("val=a+b"))                                         # needs sage.symbolic
            ('val', a + b)

        A lone list::

            sage: from sage.misc.parser import Parser, Tokenizer
            sage: p = Parser(make_var=var)                                              # needs sage.symbolic
            sage: p.p_arg(Tokenizer("[x]"))                                             # needs sage.symbolic
            [x]
        """
        cdef int token = tokens.next()
        if token == NAME and tokens.peek() == c'=':
            name = tokens.last_token_string()
            tokens.next()
            return name, self.p_expr(tokens)
        if token == c"[":
            tokens.backtrack()
            return self.p_list(tokens)
        else:
            tokens.backtrack()
            return self.p_expr(tokens)

    cdef parse_error(self, Tokenizer tokens, msg="Malformed expression"):
        raise SyntaxError(msg, tokens.s, tokens.pos)


cdef class LookupNameMaker:
    cdef object names
    cdef object fallback

    def __init__(self, names, fallback=None):
        """
        This class wraps a dictionary as a callable for use in creating names.
        It takes a dictionary of names, and an (optional) callable to use
        when the given name is not found in the dictionary.

        EXAMPLES::

            sage: # needs sage.symbolic
            sage: from sage.misc.parser import LookupNameMaker
            sage: maker = LookupNameMaker({'pi': pi}, var)
            sage: maker('pi')
            pi
            sage: maker('pi') is pi
            True
            sage: maker('a')
            a
        """
        self.names = names
        self.fallback = fallback

    def set_names(self, new_names):
        """
        TESTS::

            sage: from sage.misc.parser import LookupNameMaker
            sage: maker = LookupNameMaker({}, str)
            sage: maker.set_names({'a': x})                                             # needs sage.symbolic
            sage: maker('a') is x                                                       # needs sage.symbolic
            True
        """
        self.names = new_names

    def __call__(self, name):
        """
        TESTS::

            sage: # needs sage.symbolic
            sage: from sage.misc.parser import LookupNameMaker
            sage: maker = LookupNameMaker({'a': x}, str)
            sage: maker('a')
            x
            sage: maker('a') is x
            True
            sage: maker('b')
            'b'
        """
        try:
            return self.names[name]
        except KeyError:
            if self.fallback is not None:
                return self.fallback(name)
            raise NameError(f"Unknown variable: '{name}'")



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.symbolic
r"""
Univariate Tropical Polynomials

AUTHORS:

- Verrel Rievaldo Wijaya (2024-06): initial version

EXAMPLES::

    sage: T = TropicalSemiring(QQ, use_min=False)
    sage: R.<x> = PolynomialRing(T)
    sage: x.parent()
    Univariate Tropical Polynomial Semiring in x over Rational Field
    sage: (x + R(3)*x) * (x^2 + x)
    3*x^3 + 3*x^2
    sage: (x^2 + R(1)*x + R(-1))^2
    0*x^4 + 1*x^3 + 2*x^2 + 0*x + (-2)
    sage: (x^2 + x + R(0))^4
    0*x^8 + 0*x^7 + 0*x^6 + 0*x^5 + 0*x^4 + 0*x^3 + 0*x^2 + 0*x + 0

REFERENCES:

- [Bru2014]_
- [Fil2017]_
"""

# ****************************************************************************
#       Copyright (C) 2024 Verrel Rievaldo Wijaya <verrelrievaldo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.misc.cachefunc import cached_method
from sage.structure.unique_representation import UniqueRepresentation
from sage.structure.parent import Parent
from sage.rings.polynomial.polynomial_element_generic import Polynomial_generic_sparse


class TropicalPolynomial(Polynomial_generic_sparse):
    r"""
    A univariate tropical polynomial.

    A tropical polynomial is a polynomial whose coefficients come from
    a tropical semiring. Tropical polynomial induces a function which is
    piecewise linear, where each segment of the function has a non-negative
    integer slope. Tropical roots (zeros) of polynomial `P(x)` is defined
    as all points `x_0` for which the graph of `P(x)` change its slope.
    The difference in the slopes of the two pieces adjacent to this root
    gives the order of the root.

    The tropical polynomials are implemented with a sparse format by using
    a ``dict`` whose keys are the exponent and values the corresponding
    coefficients. Each coefficient is a tropical number.

    EXAMPLES:

    We construct a tropical polynomial semiring by defining a base
    tropical semiring and then inputting it to :class:`PolynomialRing`::

        sage: T = TropicalSemiring(QQ, use_min=False)
        sage: R.<x> = PolynomialRing(T); R
        Univariate Tropical Polynomial Semiring in x over Rational Field
        sage: R.0
        0*x

    One way to construct an element is to provide a list or tuple of
    coefficients. Another way to define an element is to write a polynomial
    equation with each coefficient converted to the semiring::

        sage: p1 = R([1,4,None,0]); p1
        0*x^3 + 4*x + 1
        sage: p2 = R(1)*x^2 + R(2)*x + R(3); p2
        1*x^2 + 2*x + 3

    We can do some basic arithmetic operations for these tropical polynomials.
    Remember that numbers given have to be in the tropical semiring.
    If not, then it will raise an error::

        sage: p1 + p2
        0*x^3 + 1*x^2 + 4*x + 3
        sage: p1 * p2
        1*x^5 + 2*x^4 + 5*x^3 + 6*x^2 + 7*x + 4
        sage: 2 * p1
        Traceback (most recent call last):
        ...
        ArithmeticError: cannot negate any non-infinite element
        sage: T(2) * p1
        2*x^3 + 6*x + 3
        sage: p1(3)
        Traceback (most recent call last):
        ...
        TypeError: no common canonical parent for objects with parents:
        'Tropical semiring over Rational Field' and 'Integer Ring'
        sage: p1(T(3))
        9

    We can find all tropical roots of a tropical polynomial, counted
    with their multiplicities::

        sage: p1.roots()
        [-3, 2, 2]
        sage: p2.roots()
        [1, 1]

    Even though every tropical polynomials have tropical roots, this does not
    neccessarily means it can be factored into its linear factors::

        sage: p1.factor()
        (0) * (0*x^3 + 4*x + 1)
        sage: p2.factor()
        (1) * (0*x + 1)^2

    Every tropical polynomial `p(x)` have a corresponding unique tropical
    polynomial `\bar{p}(x)` with the same roots that can be factored. We
    call `\bar{p}(x)` the tropical polynomial split form of `p(x)`::

        sage: p1.split_form()
        0*x^3 + 2*x^2 + 4*x + 1
        sage: p2.split_form()
        1*x^2 + 2*x + 3

    Every tropical polynomial induce a piecewise linear function that can be
    invoked in the following way::

        sage: p1.piecewise_function()
        piecewise(x|-->1 on (-oo, -3], x|-->x + 4 on (-3, 2), x|-->3*x on [2, +oo); x)
        sage: p1.plot()                                                                 # needs sage.plot
        Graphics object consisting of 1 graphics primitive

    .. PLOT::
        :width: 300 px

        T = TropicalSemiring(QQ, use_min=False)
        R = PolynomialRing(T, 'x')
        p1 = R([1,4,None,0])
        sphinx_plot(p1.plot())

    ::

        sage: p2.piecewise_function()
        piecewise(x|-->3 on (-oo, 1], x|-->2*x + 1 on (1, +oo); x)
        sage: p2.plot()                                                                 # needs sage.plot
        Graphics object consisting of 1 graphics primitive

    .. PLOT::
        :width: 300 px

        T = TropicalSemiring(QQ, use_min=False)
        R = PolynomialRing(T, 'x')
        x = R.gen()
        p2 = R(1)*x**2 + R(2)*x + R(3)
        sphinx_plot(plot(p2, xmin=-1, xmax=3))

    TESTS:

    There is no subtraction for tropical polynomials because element in
    tropical semiring doesn't necessarily have additive inverse::

        sage: -p1
        Traceback (most recent call last):
        ...
        ArithmeticError: cannot negate any non-infinite element
    """
    def roots(self):
        r"""
        Return the list of all tropical roots of ``self``, counted with
        multiplicity.

        OUTPUT: a list of tropical numbers

        ALGORITHM:

        For each pair of monomials in the polynomial, we find the point
        where their values are equal.  This is the same as solving the
        equation `c_1 + a_1 \cdot x = c_2 + a_2 \cdot x` for `x`, where
        `(c_1, a_1)` and `(c_2, a_2)` are the coefficients and exponents
        of monomials.

        The solution to this equation is `x = (c_2-c_1)/(a_1-a_2)`. We
        substitute this `x` to each monomials in polynomial and check if
        the maximum (minimum) is achieved by the previous two monomials.
        If it is, then `x` is the root of tropical polynomial. In this
        case, the order of the root at `x` is the maximum of `|i-j|` for
        all possible pairs `i,j` which realise this maximum (minimum).

        EXAMPLES::

            sage: T = TropicalSemiring(QQ, use_min=True)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([5,4,1,0,2,4,3]); p1
            3*x^6 + 4*x^5 + 2*x^4 + 0*x^3 + 1*x^2 + 4*x + 5
            sage: p1.roots()
            [-1, -1, -1, 1, 2, 2]

        There will be no tropical root for constant polynomial. Additionaly,
        for a monomial, the tropical root is assumed to be the additive
        identity of its base tropical semiring::

            sage: p2 = R(2)
            sage: p2.roots()
            []
            sage: p3 = x^3
            sage: p3.roots()
            [+infinity, +infinity, +infinity]
        """
        from itertools import combinations
        tropical_roots = []
        data = self.monomial_coefficients()
        R = self.parent().base()
        if len(data) == 1:
            exponent = next(iter(data))
            if not exponent:
                return tropical_roots
            return [R.zero()] * exponent
        dict_root = {}
        dict_coeff = {i: c.lift() for i, c in data.items()}
        for comb in combinations(dict_coeff, 2):
            index1, index2 = comb[0], comb[1]
            root = (dict_coeff[index1]-dict_coeff[index2]) / (index2-index1)
            val_root = dict_coeff[index1] + index1*root
            check_maks = True
            for key in dict_coeff:
                if key not in comb:
                    val = dict_coeff[key] + key*root
                    if R._use_min:
                        if val < val_root:
                            check_maks = False
                            break
                    elif val > val_root:
                        check_maks = False
                        break
            if check_maks:
                order = abs(index1-index2)
                if root not in dict_root:
                    dict_root[root] = order
                elif order > dict_root[root]:
                    dict_root[root] = order
        for root in dict_root:
            tropical_roots += [root] * dict_root[root]
        return sorted(tropical_roots)

    def split_form(self):
        r"""
        Return the tropical polynomial which has the same roots as ``self``
        but which can be reduced to its linear factors.

        If a tropical polynomial has roots at `x_1, x_2, \ldots, x_n`, then
        its split form is the tropical product of linear terms of the form
        `(x + x_i)` for all `i=1,2,\ldots,n`.

        OUTPUT: :class:`TropicalPolynomial`

        EXAMPLES::

            sage: T = TropicalSemiring(QQ, use_min=True)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([5,4,1,0,2,4,3]); p1
            3*x^6 + 4*x^5 + 2*x^4 + 0*x^3 + 1*x^2 + 4*x + 5
            sage: p1.split_form()
            3*x^6 + 2*x^5 + 1*x^4 + 0*x^3 + 1*x^2 + 3*x + 5

        ::

            sage: T = TropicalSemiring(QQ, use_min=False)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([5,4,1,0,2,4,3])
            sage: p1.split_form()
            3*x^6 + 4*x^5 + 21/5*x^4 + 22/5*x^3 + 23/5*x^2 + 24/5*x + 5

        TESTS:

        We verify that the roots of tropical polynomial and its split form
        is really the same::

            sage: p1.roots() == p1.split_form().roots()
            True
        """
        roots = self.roots()
        R = self.parent()
        poly = R(self.monomial_coefficients()[self.degree()].lift())
        for root in roots:
            linear = R([root, 0])
            poly *= linear
        return poly

    def factor(self):
        r"""
        Return the factorization of ``self`` into its tropical linear factors.

        Note that the factor `x - x_0` in classical algebra gets transformed
        to the factor `x + x_0`, since the root of the tropical polynomial
        `x + x_0` is `x_0` and not `-x_0`. However, not every tropical
        polynomial can be factored.

        OUTPUT: :class:'Factorization'

        EXAMPLES::

            sage: T = TropicalSemiring(QQ, use_min=True)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([6,3,1,0]); p1
            0*x^3 + 1*x^2 + 3*x + 6
            sage: factor(p1)
            (0) * (0*x + 1) * (0*x + 2) * (0*x + 3)

        Such factorization is not always possible::

            sage: p2 = R([4,4,2]); p2
            2*x^2 + 4*x + 4
            sage: p2.factor()
            (2) * (0*x^2 + 2*x + 2)

        TESTS:

        The factorization for a constant::

            sage: p3 = R(3)
            sage: p3.factor()
            (3) * 0
        """
        from sage.structure.factorization import Factorization
        unit = self.monomial_coefficients()[self.degree()]
        if self != self.split_form() or not self.roots():
            factor = [(self * self.parent(-unit.lift()), 1)]
            return Factorization(factor, unit=unit)

        R = self.parent()
        roots_order = {}
        for root in self.roots():
            if root in roots_order:
                roots_order[root] += 1
            else:
                roots_order[root] = 1
        factors = [(R([root, 0]), roots_order[root])
                   for root in roots_order]
        return Factorization(factors, unit=unit)

    def piecewise_function(self):
        r"""
        Return the tropical polynomial function of ``self``.

        The function is a piecewise linear function with the domains are
        divided by the roots. First we convert each term of polynomial to
        its corresponding linear function. Next, we must determine which
        term achieves the minimum (maximum) at each interval.

        OUTPUT: a piecewise function

        EXAMPLES::

            sage: T = TropicalSemiring(QQ, use_min=False)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([4,2,1,3]); p1
            3*x^3 + 1*x^2 + 2*x + 4
            sage: p1.piecewise_function()
            piecewise(x|-->4 on (-oo, 1/3], x|-->3*x + 3 on (1/3, +oo); x)

        A constant tropical polynomial will result in a constant function::

            sage: p2 = R(3)
            sage: p2.piecewise_function()
            3

        A monomial will resulted in a linear function::

            sage: p3 = R(1)*x^3
            sage: p3.piecewise_function()
            3*x + 1
        """
        from sage.symbolic.ring import SR
        from sage.functions.piecewise import piecewise
        from sage.sets.real_set import RealSet

        x = SR.var('x')
        data = self.monomial_coefficients()
        R = self.parent().base()
        if not self.roots():
            f = data[0].lift()
            return f

        if len(data) == 1:
            gradient = list(data)[0]
            intercept = data[gradient].lift()
            f = intercept + gradient*x
            return f

        unique_root = sorted(set(self.roots()))
        pieces = []
        domain = []
        for i in range(len(unique_root) + 1):
            if i == 0:
                test_number = R(unique_root[i] - 1)
            elif i == len(unique_root):
                test_number = R(unique_root[i-1] + 1)
            else:
                test_number = R((unique_root[i]+unique_root[i-1]) / 2)
            terms = {i: c * test_number**i for i, c in data.items()}
            if R._use_min:
                critical = min(terms.values())
            else:
                critical = max(terms.values())
            found_key = None
            for key, value in terms.items():
                if value == critical:
                    found_key = key
                    break
            gradient = found_key
            intercept = data[found_key].lift()

            # To make sure all roots is included in the domain
            if i == 0:
                interval = RealSet.unbounded_below_closed(unique_root[i])
                piecewise_linear = (interval, intercept + gradient*x)
                domain.append(interval)
            elif i == len(unique_root):
                if domain[i-1][0].upper_closed():
                    interval = RealSet.unbounded_above_open(unique_root[i-1])
                else:
                    interval = RealSet.unbounded_above_closed(unique_root[i-1])
                piecewise_linear = (interval, intercept + gradient*x)
                domain.append(interval)
            else:
                if domain[i-1][0].upper_closed():
                    interval = RealSet((unique_root[i-1], unique_root[i]))
                else:
                    interval = RealSet([unique_root[i-1], unique_root[i]])
                piecewise_linear = (interval, intercept + gradient*x)
                domain.append(interval)
            pieces.append(piecewise_linear)

        f = piecewise(pieces)
        return f

    def plot(self, xmin=None, xmax=None):
        r"""
        Return the plot of ``self``, which is the tropical polynomial
        function we get from ``self.piecewise_function()``.

        INPUT:

        - ``xmin`` -- (optional) real number
        - ``xmax`` -- (optional) real number

        OUTPUT:

        If ``xmin`` and ``xmax`` is given, then it return a plot of
        piecewise linear function of ``self`` with the axes start from
        ``xmin`` to ``xmax``. Otherwise, the domain will start from the
        the minimum root of ``self`` minus 1 to maximum root of ``self``
        plus 1. If the function of ``self`` is constant or linear, then
        the default domain will be `[-1,1]`.

        EXAMPLES:

        If the tropical semiring use a max-plus algebra, then the graph
        will be of piecewise linear convex function::

            sage: T = TropicalSemiring(QQ, use_min=False)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([4,2,1,3]); p1
            3*x^3 + 1*x^2 + 2*x + 4
            sage: p1.roots()
            [1/3, 1/3, 1/3]
            sage: p1.plot()                                                             # needs sage.plot
            Graphics object consisting of 1 graphics primitive

        .. PLOT::
            :width: 300 px

            T = TropicalSemiring(QQ, use_min=False)
            R = PolynomialRing(T, 'x')
            p1 = R([4,2,1,3])
            sphinx_plot(p1.plot())

        A different result will be obtained if the tropical semiring employs
        a min-plus algebra. Rather, a graph of the piecewise linear concave
        function will be obtained::

            sage: T = TropicalSemiring(QQ, use_min=True)
            sage: R.<x> = PolynomialRing(T)
            sage: p1 = R([4,2,1,3])
            sage: p1.roots()
            [-2, 1, 2]
            sage: p1.plot()                                                             # needs sage.plot
            Graphics object consisting of 1 graphics primitive

        .. PLOT::
            :width: 300 px

            T = TropicalSemiring(QQ, use_min=True)
            R = PolynomialRing(T, 'x')
            p1 = R([4,2,1,3])
            sphinx_plot(p1.plot())

        TESTS:

        If ``xmin`` or ``xmax`` is given as an input, then the others also
        have to be given. Otherwise it will raise an error::

            sage: plot(p1, 5)                                                           # needs sage.plot
            Traceback (most recent call last):
            ...
            ValueError: expected 2 inputs for xmin and xmax, but got 1

        Error also occured when ``xmin`` is greater or equal than``xmax``::

            sage: plot(p1, 5, 3)                                                        # needs sage.plot
            Traceback (most recent call last):
            ...
            ValueError: xmin = 5 should be less than xmax = 3
        """
        from sage.plot.plot import plot
        f = self.piecewise_function()
        if (xmin is None) and (xmax is None):
            roots = sorted(self.roots())
            if (not roots) or (self.parent().base().zero() in roots):
                return plot(f, xmin=-1, xmax=1)
            else:
                return plot(f, xmin=roots[0]-1, xmax=roots[-1]+1)
        elif xmin is None or xmax is None:
            raise ValueError("expected 2 inputs for xmin and xmax, but got 1")
        elif xmin >= xmax:
            raise ValueError(f"xmin = {xmin} should be less than xmax = {xmax}")
        return plot(f, xmin=xmin, xmax=xmax)

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R.<x> = PolynomialRing(T)
            sage: R([-3,-1,2,-1])
            (-1)*x^3 + 2*x^2 + (-1)*x + (-3)
        """
        import re
        if not self.monomial_coefficients():
            return str(self.parent().base().zero())

        def replace_negatives(expr):
            modified_expr = re.sub(r'(-\d+/\d+|-\d+)', r'(\1)', expr)
            return modified_expr

        s = super()._repr()
        v = self.parent().variable_name()
        if s[0] == v:
            s = "1*" + s
        s = s.replace(" - ", " + -")
        s = s.replace(" + "+v, " + 1*"+v)
        s = s.replace("-"+v, "-1*"+v)
        s = replace_negatives(s)
        return s

    def _latex_(self):
        r"""
        Return a latex representation of ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: p1 = R([-1,2,None,-3])
            sage: latex(p1)
            \left(-3\right) x^{3} + 2 x + \left(-1\right)
        """
        s = " "
        coeffs = self.list(copy=False)
        m = len(coeffs)
        name = self.parent().latex_variable_names()[0]
        for n in reversed(range(m)):
            x = coeffs[n]
            x = x._latex_()
            if x != self.parent().base().zero()._latex_():
                if n != m-1:
                    s += " + "
                if x.find("-") == 0:
                    x = "\\left(" + x + "\\right)"
                if n > 1:
                    v = f"|{name}^{{{n}}}"
                elif n == 1:
                    v = f"|{name}"
                else:
                    v = ""
                s += f"{x} {v}"
        s = s.replace("|", "")
        if s == " ":
            return self.parent().base().zero()._latex_()
        return s[1:].lstrip().rstrip()


class TropicalPolynomialSemiring(UniqueRepresentation, Parent):
    r"""
    The semiring of univariate tropical polynomials.

    This is the commutative semiring consisting of finite linear
    combinations of tropical monomials under (tropical)
    addition and multiplication with the identity element
    being `+\infty` (or `-\infty` depending on whether the
    base tropical semiring is using min-plus or max-plus algebra).

    EXAMPLES::

        sage: T = TropicalSemiring(QQ)
        sage: R.<x> = PolynomialRing(T)
        sage: f = T(1)*x
        sage: g = T(2)*x
        sage: f + g
        1*x
        sage: f * g
        3*x^2
        sage: f + R.zero() == f
        True
        sage: f * R.zero() == R.zero()
        True
        sage: f * R.one() == f
        True
    """
    @staticmethod
    def __classcall_private__(cls, base_semiring, names):
        """
        Ensures the names parameter is a tuple.

        EXAMPLES::

            sage: from sage.rings.semirings.tropical_polynomial import TropicalPolynomialSemiring
            sage: T = TropicalSemiring(ZZ)
            sage: TPS = TropicalPolynomialSemiring
            sage: TPS(T, 'x') == TPS(T, ('x'))
            True
        """
        if isinstance(names, str):
            names = (names,)
        return super().__classcall__(cls, base_semiring, tuple(names))

    def __init__(self, base_semiring, names):
        r"""
        Initialize ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: TestSuite(R).run()

        TESTS::

            sage: from sage.rings.semirings.tropical_polynomial import TropicalPolynomialSemiring
            sage: TropicalPolynomialSemiring(ZZ, names='x')
            Traceback (most recent call last):
            ...
            ValueError: Integer Ring is not a tropical semiring
        """
        from sage.categories.semirings import Semirings
        from sage.rings.semirings.tropical_semiring import TropicalSemiring
        if not isinstance(base_semiring, TropicalSemiring):
            raise ValueError(f"{base_semiring} is not a tropical semiring")
        Parent.__init__(self, base=base_semiring, names=names, category=Semirings())

    Element = TropicalPolynomial

    def _element_constructor_(self, x=None, check=True):
        """
        Convert ``x`` into ``self``, possibly non-canonically.

        INPUT:

        - ``x`` -- a list or tuple of coefficients, a polynomial, or a
          dictionary

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: R([1,2,3])
            3*x^2 + 2*x + 1
            sage: S.<x> = PolynomialRing(QQ)
            sage: R(x^2 - x + 1)
            1*x^2 + (-1)*x + 1

        If ``x`` is a tropical polynomial from different semiring, then it
        will converted to constant::

            sage: T = TropicalSemiring(QQ)
            sage: R.<x> = PolynomialRing(T)
            sage: S.<y> = PolynomialRing(T)
            sage: p1 = R(y); p1
            0*y
            sage: p1.parent()
            Univariate Tropical Polynomial Semiring in x over Rational Field
            sage: p1.degree()
            0
        """
        if isinstance(x, (list, tuple)):
            for i, coeff in enumerate(x):
                if coeff == 0:
                    x[i] = self.base().one()
        elif isinstance(x, TropicalPolynomial):
            if x.parent() is not self:
                x = {0: x}
                check = False
        return self.element_class(self, x, check=check)

    @cached_method
    def one(self):
        """
        Return the multiplicative identity of ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: R.one()
            0
        """
        return self.element_class(self, [self.base().one()])

    @cached_method
    def zero(self):
        """
        Return the additive identity of ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: R.zero()
            +infinity
        """
        return self.element_class(self, [self.base().zero()])

    def _repr_(self):
        """
        Return a string representation of ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(ZZ)
            sage: R.<abc> = PolynomialRing(T); R
            Univariate Tropical Polynomial Semiring in abc over Integer Ring
        """
        return (f"Univariate Tropical Polynomial Semiring in {self.variable_name()}"
                f" over {self.base_ring().base_ring()}")

    def gen(self, n=0):
        """
        Return the indeterminate generator of ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R.<abc> = PolynomialRing(T)
            sage: R.gen()
            0*abc

        TESTS::

            sage: R.gen(2)
            Traceback (most recent call last):
            ...
            IndexError: generator n not defined
        """
        if n != 0:
            raise IndexError("generator n not defined")
        return self.gens()[n]

    @cached_method
    def gens(self):
        """
        Return a tuple whose entries are the generators for ``self``.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R.<abc> = PolynomialRing(T)
            sage: R.gens()
            (0*abc,)
        """
        R = self.base()
        return (self.element_class(self, [R.zero(), R.one()]),)

    def ngens(self):
        """
        Return the number of generators of ``self``, which is 1
        since it is a univariate polynomial ring.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: R.ngens()
            1
        """
        from sage.rings.integer_ring import ZZ
        return ZZ.one()

    def random_element(self, degree=(-1, 2), monic=False, *args, **kwds):
        r"""
        Return a random tropical polynomial of given degrees (bounds).

        OUTPUT: :class:`TropicalPolynomial`

        .. SEEALSO::

            :meth:`sage.rings.polynomial.polynomial_ring.PolynomialRing_general.random_element`

        EXAMPLES:

        Tropical polynomial over an integer with each coefficient bounded
        between ``x`` and ``y``::

            sage: T = TropicalSemiring(ZZ)
            sage: R = PolynomialRing(T, 'x')
            sage: f = R.random_element(8, x=3, y=10)
            sage: f.degree()
            8
            sage: f.parent() is R
            True
            sage: all(a >= T(3) for a in f.coefficients())
            True
            sage: all(a < T(10) for a in f.coefficients())
            True

        If a tuple of two integers is provided for the ``degree`` argument,
        a polynomial is selected with degrees within that range::

            sage: all(R.random_element(degree=(1, 5)).degree() in range(1, 6) for _ in range(10^3))
            True

        Note that the zero polynomial (`\pm \infty`) has degree `-1`.
        To include it, set the minimum degree to `-1`::

            sage: while R.random_element(degree=(-1,2), x=-1, y=1) != R.zero():
            ....:     pass

        Monic polynomials are chosen among all monic polynomials with degree
        between the given ``degree`` argument::

            sage: all(R.random_element(degree=(-1, 2), monic=True).is_monic() for _ in range(10^3))
            True
        """
        from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
        R = PolynomialRing(self.base().base_ring(), self.variable_names())
        f = R.random_element(degree=degree, monic=monic, *args, **kwds)
        new_dict = f.monomial_coefficients()
        if monic:
            new_dict[f.degree()] = 0
        return self.element_class(self, new_dict)

    def is_sparse(self):
        """
        Return ``True`` to indicate that the objects are sparse polynomials.

        EXAMPLES::

            sage: T = TropicalSemiring(QQ)
            sage: R = PolynomialRing(T, 'x')
            sage: R.is_sparse()
            True
        """
        return True

    def interpolation(self, points):
        """
        Return a tropical polynomial with its function is a linear
        interpolation of point in ``points`` if possible.

        If there is only one point, then it will give a constant polynomial.
        Because we are using linear interpolation, each point is actually
        a root of the resulted tropical polynomial.

        INPUT:

        - ``points`` -- a list of tuples ``(x, y)``

        OUTPUT: :class:`TropicalPolynomial`

        EXAMPLES::

            sage: T = TropicalSemiring(QQ, use_min=True)
            sage: R = PolynomialRing(T, 'x')
            sage: points = [(-2,-3),(1,3),(2,4)]
            sage: p1 = R.interpolation(points); p1
            1*x^2 + 2*x + 4
            sage: p1.plot()                                                             # needs sage.plot
            Graphics object consisting of 1 graphics primitive

        .. PLOT::
            :width: 300 px

            T = TropicalSemiring(QQ, use_min=True)
            R = PolynomialRing(T, 'x')
            points = [(-2,-3),(1,3),(2,4)]
            p1 = R.interpolation(points)
            sphinx_plot(p1.plot())

        ::

            sage: T = TropicalSemiring(QQ, use_min=False)
            sage: R = PolynomialRing(T,'x')
            sage: points = [(0,0),(1,1),(2,4)]
            sage: p1 = R.interpolation(points); p1
            (-2)*x^3 + (-1)*x^2 + 0*x + 0
            sage: p1.plot()                                                             # needs sage.plot
            Graphics object consisting of 1 graphics primitive

        .. PLOT::
            :width: 300 px

            T = TropicalSemiring(QQ, use_min=False)
            R = PolynomialRing(T, 'x')
            points = [(0,0),(1,1),(2,4)]
            p1 = R.interpolation(points)
            sphinx_plot(p1.plot())

        TESTS:

        Every piecewise linear component of tropical polynomial function
        has to have an integer slope::

            sage: T = TropicalSemiring(QQ, use_min=False)
            sage: R = PolynomialRing(T,'x')
            sage: points = [(0,0),(2,3)]
            sage: R.interpolation(points)
            Traceback (most recent call last):
            ...
            ValueError: the slope is not an integer

        For max-plus algebra, the slope of the componenets has to be
        increasing as we move from left to right. Conversely for min-plus
        algebra, the slope of the componenets has to be decreasing from
        left to right::

            sage: T = TropicalSemiring(QQ, use_min=False)
            sage: R = PolynomialRing(T,'x')
            sage: points = [(-2,-3),(1,3),(2,4)]
            sage: R.interpolation(points)
            Traceback (most recent call last):
            ...
            ValueError: can not interpolate these points
        """
        points = sorted(points, key=lambda point: point[0])
        all_slope = [0]
        roots = {}
        R = self.base()
        if R._use_min:
            point_order = range(len(points)-1, 0, -1)
        else:
            point_order = range(len(points)-1)
        for i in point_order:
            if R._use_min:
                slope = (points[i-1][1]-points[i][1]) / (points[i-1][0]-points[i][0])
            else:
                slope = (points[i+1][1]-points[i][1]) / (points[i+1][0]-points[i][0])
            if not slope.is_integer():
                raise ValueError("the slope is not an integer")
            if slope < all_slope[-1]:
                raise ValueError("can not interpolate these points")
            elif slope > all_slope[-1]:
                order = slope - all_slope[-1]
                all_slope.append(slope)
                roots[points[i][0]] = order

        if len(all_slope) == 1:  # constant polynomial
            return self(points[0][1])

        result = self.one()
        for root, order in roots.items():
            result *= self([root, 0])**order
        test_value = result(R(points[0][0]))
        unit = R(points[0][1] - test_value.lift())
        result *= unit
        return result

    @classmethod
    def _implementation_names(cls, implementation, base_ring, sparse):
        """
        Return the only implementation currently available for this semiring,
        which is ``[None]``.

        EXAMPLES::

            sage: from sage.rings.semirings.tropical_polynomial import TropicalPolynomialSemiring
            sage: T = TropicalSemiring(QQ)
            sage: TropicalPolynomialSemiring._implementation_names(None, T, True)
            [None]
        """
        names = [None]
        assert isinstance(names, list)
        assert implementation in names
        return names



# sage_setup: distribution = sagemath-categories
"""
Univariate Polynomials over domains and fields

AUTHORS:

- William Stein: first version
- Martin Albrecht: Added singular coercion.
- David Harvey: split off polynomial_integer_dense_ntl.pyx (2007-09)
- Robert Bradshaw: split off polynomial_modn_dense_ntl.pyx (2007-09)

TESTS:

We test coercion in a particularly complicated situation::

    sage: W.<w> = QQ['w']
    sage: WZ.<z> = W['z']
    sage: m = matrix(WZ, 2, 2, [1, z, z, z^2])                                          # needs sage.modules
    sage: a = m.charpoly()                                                              # needs sage.modules
    sage: R.<x> = WZ[]
    sage: R(a)                                                                          # needs sage.modules
    x^2 + (-z^2 - 1)*x
"""

# ****************************************************************************
#       Copyright (C) 2007 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.rings.polynomial.polynomial_element import Polynomial, Polynomial_generic_dense, Polynomial_generic_dense_inexact
from sage.structure.element import IntegralDomainElement, EuclideanDomainElement

from sage.rings.polynomial.polynomial_singular_interface import Polynomial_singular_repr

try:
    from sage.libs.pari.all import pari_gen
except ImportError:
    pari_gen = ()

from sage.structure.element import coerce_binop, parent
from sage.structure.factorization import Factorization
from sage.structure.richcmp import richcmp, richcmp_item, rich_to_bool, rich_to_bool_sgn

from sage.rings.infinity import infinity, Infinity
from sage.rings.integer_ring import ZZ
from sage.rings.integer import Integer
from sage.rings.padics.precision_error import PrecisionError


class Polynomial_generic_sparse(Polynomial):
    """
    A generic sparse polynomial.

    The ``Polynomial_generic_sparse`` class defines functionality for sparse
    polynomials over any base ring. A sparse polynomial is represented using a
    dictionary which maps each exponent to the corresponding coefficient. The
    coefficients must never be zero.

    EXAMPLES::

        sage: R.<x> = PolynomialRing(PolynomialRing(QQ, 'y'), sparse=True)
        sage: f = x^3 - x + 17
        sage: type(f)
        <class 'sage.rings.polynomial.polynomial_ring.PolynomialRing_integral_domain_with_category.element_class'>
        sage: loads(f.dumps()) == f
        True

    A more extensive example::

        sage: # needs sage.libs.pari
        sage: A.<T> = PolynomialRing(Integers(5), sparse=True)
        sage: f = T^2 + 1; B = A.quo(f)
        sage: C.<s> = PolynomialRing(B)
        sage: C
        Univariate Polynomial Ring in s over Univariate Quotient Polynomial Ring in Tbar
         over Ring of integers modulo 5 with modulus T^2 + 1
        sage: s + T
        s + Tbar
        sage: (s + T)**2
        s^2 + 2*Tbar*s + 4
    """
    def __init__(self, parent, x=None, check=True, is_gen=False, construct=False):
        """
        TESTS::

            sage: PolynomialRing(RIF, 'z', sparse=True)([RIF(-1, 1), RIF(-1,1)])                    # needs sage.rings.real_interval_field
            0.?*z + 0.?
            sage: PolynomialRing(RIF, 'z', sparse=True)((RIF(-1, 1), RIF(-1,1)))                    # needs sage.rings.real_interval_field
            0.?*z + 0.?
            sage: PolynomialRing(CIF, 'z', sparse=True)([CIF(RIF(-1,1), RIF(-1,1)), RIF(-1,1)])     # needs sage.rings.complex_interval_field
            0.?*z + 0.? + 0.?*I
        """
        Polynomial.__init__(self, parent, is_gen=is_gen)
        if x is None:
            self.__coeffs = {}
            return
        R = parent.base_ring()
        if isinstance(x, Polynomial):
            if x.parent() == self.parent():
                x = x.monomial_coefficients()
            elif x.parent() == R:
                x = {0: x}
            else:
                w = {}
                for n, c in x.monomial_coefficients().items():
                    w[n] = R(c)
                # The following line has been added in github issue #9944.
                # Apparently, the "else" case has never occurred before.
                x = w
        elif isinstance(x, (list, tuple)):
            x = {i: c for i, c in enumerate(x) if c}
        elif isinstance(x, pari_gen):
            y = {}
            for i in range(len(x)):
                y[i] = R(x[i])
            x = y
            check = True
        elif not isinstance(x, dict):
            x = {0: x}   # constant polynomials
        if check:
            self.__coeffs = {}
            for i, z in x.items():
                self.__coeffs[i] = R(z)
        else:
            self.__coeffs = x
        if check:
            self.__normalize()

    def monomial_coefficients(self):
        """
        Return a new copy of the dict of the underlying
        elements of ``self``.

        EXAMPLES::

            sage: R.<w> = PolynomialRing(Integers(8), sparse=True)
            sage: f = 5 + w^1997 - w^10000; f
            7*w^10000 + w^1997 + 5
            sage: d = f.monomial_coefficients(); d
            {0: 5, 1997: 1, 10000: 7}
            sage: d[0] = 10
            sage: f.monomial_coefficients()
            {0: 5, 1997: 1, 10000: 7}

        ``dict`` is an alias::

            sage: f.dict()
            {0: 5, 1997: 1, 10000: 7}
        """
        return dict(self.__coeffs)

    dict = monomial_coefficients

    def coefficients(self, sparse=True):
        """
        Return the coefficients of the monomials appearing in ``self``.

        EXAMPLES::

            sage: R.<w> = PolynomialRing(Integers(8), sparse=True)
            sage: f = 5 + w^1997 - w^10000; f
            7*w^10000 + w^1997 + 5
            sage: f.coefficients()
            [5, 1, 7]

        TESTS:

        Check that all coefficients are in the base ring::

            sage: S.<x> = PolynomialRing(QQ, sparse=True)
            sage: f = x^4
            sage: all(c.parent() is QQ for c in f.coefficients(False))
            True
        """
        if sparse:
            return [self.__coeffs[e] for e in self.exponents()]
        else:
            zero = self.parent().base_ring().zero()
            return [self.__coeffs[i] if i in self.__coeffs else zero
                    for i in range(self.degree() + 1)]

    def exponents(self):
        """
        Return the exponents of the monomials appearing in ``self``.

        EXAMPLES::

            sage: R.<w> = PolynomialRing(Integers(8), sparse=True)
            sage: f = 5 + w^1997 - w^10000; f
            7*w^10000 + w^1997 + 5
            sage: f.exponents()
            [0, 1997, 10000]
        """
        return sorted(self.__coeffs)

    def valuation(self, p=None):
        """
        Return the valuation of ``self``.

        EXAMPLES::

            sage: # needs sage.rings.finite_rings
            sage: R.<w> = PolynomialRing(GF(9, 'a'), sparse=True)
            sage: f = w^1997 - w^10000
            sage: f.valuation()
            1997
            sage: R(19).valuation()
            0
            sage: R(0).valuation()
            +Infinity
        """
        if not self.__coeffs:
            return infinity

        if p is infinity:
            return -self.degree()

        if p is not None:
            raise NotImplementedError("input p is not support for sparse polynomials")

        return ZZ(min(self.__coeffs))

    def _derivative(self, var=None):
        """
        Return the formal derivative of this polynomial with respect to
        the given variable.

        If ``var`` is ``None`` or is the generator of this ring, the
        derivative is with respect to the generator. Otherwise,
        _derivative(var) is called recursively for each coefficient of
        this polynomial.

        .. SEEALSO:: :meth:`.derivative`

        EXAMPLES::

            sage: R.<w> = PolynomialRing(ZZ, sparse=True)
            sage: f = R(range(9)); f
            8*w^8 + 7*w^7 + 6*w^6 + 5*w^5 + 4*w^4 + 3*w^3 + 2*w^2 + w
            sage: f._derivative()
            64*w^7 + 49*w^6 + 36*w^5 + 25*w^4 + 16*w^3 + 9*w^2 + 4*w + 1
            sage: f._derivative(w)
            64*w^7 + 49*w^6 + 36*w^5 + 25*w^4 + 16*w^3 + 9*w^2 + 4*w + 1

            sage: R.<x> = PolynomialRing(ZZ)
            sage: S.<y> = PolynomialRing(R, sparse=True)
            sage: f = x^3*y^4
            sage: f._derivative()
            4*x^3*y^3
            sage: f._derivative(y)
            4*x^3*y^3
            sage: f._derivative(x)
            3*x^2*y^4

        Check that :issue:`28187` is fixed::

            sage: R = PolynomialRing(ZZ, 't', sparse=True)
            sage: t, u = var('t, u')                                                    # needs sage.symbolic
            sage: R.gen()._derivative(t)                                                # needs sage.symbolic
            1
            sage: R.gen()._derivative(u)                                                # needs sage.symbolic
            Traceback (most recent call last):
            ...
            ValueError: cannot differentiate with respect to u
        """
        P = self.parent()
        if var is not None and var != P.gen():
            try:
                # call _derivative() recursively on coefficients
                return P({n:self.__coeffs[n]._derivative(var)
                            for n in self.__coeffs})
            except AttributeError:
                raise ValueError('cannot differentiate with respect to {}'.format(var))

        # compute formal derivative with respect to generator
        d = {}
        for n, c in self.__coeffs.items():
            d[n-1] = n*c
        if -1 in d:
            del d[-1]
        return P(d)

    def integral(self, var=None):
        """
        Return the integral of this polynomial.

        By default, the integration variable is the variable of the
        polynomial.

        Otherwise, the integration variable is the optional parameter ``var``

        .. NOTE::

            The integral is always chosen so that the constant term is 0.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (1 + 3*x^10 - 2*x^100).integral()
            -2/101*x^101 + 3/11*x^11 + x

        TESTS:

        Check that :issue:`18600` is fixed::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (x^2^100).integral()
            1/1267650600228229401496703205377*x^1267650600228229401496703205377

        Check the correctness when the base ring is a polynomial ring::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: S.<t> = PolynomialRing(R, sparse=True)
            sage: (x*t+1).integral()
            1/2*x*t^2 + t
            sage: (x*t+1).integral(x)
            1/2*x^2*t + x

        Check the correctness when the base ring is not an integral domain::

            sage: R.<x> = PolynomialRing(Zmod(4), sparse=True)
            sage: (x^4 + 2*x^2  + 3).integral()
            x^5 + 2*x^3 + 3*x
            sage: x.integral()
            Traceback (most recent call last):
            ...
            ZeroDivisionError: inverse of Mod(2, 4) does not exist
        """
        R = self.parent()
        # TODO:
        # calling the coercion model bin_op is much more accurate than using the
        # true division (which is bypassed by polynomials). But it does not work
        # in all cases!!
        from sage.structure.element import coercion_model as cm
        import operator
        try:
            Q = cm.bin_op(R.one(), ZZ.one(), operator.truediv).parent()
        except TypeError:
            F = (R.base_ring().one()/ZZ.one()).parent()
            Q = R.change_ring(F)

        if var is not None and var != R.gen():
            return Q({k:v.integral(var) for k,v in self.__coeffs.items()}, check=False)

        return Q({ k+1:v/(k+1) for k,v in self.__coeffs.items()}, check=False)

    def _dict_unsafe(self):
        """
        Return unsafe access to the underlying dictionary of coefficients.

        ** DO NOT use this, unless you really really know what you are doing. **

        EXAMPLES::

            sage: R.<w> = PolynomialRing(ZZ, sparse=True)
            sage: f = w^15 - w*3; f
            w^15 - 3*w
            sage: d = f._dict_unsafe(); d
            {1: -3, 15: 1}
            sage: d[1] = 10; f
            w^15 + 10*w
        """
        return self.__coeffs

    def _repr(self, name=None):
        r"""
        EXAMPLES::

            sage: # needs sage.rings.complex_double sage.symbolic
            sage: R.<w> = PolynomialRing(CDF, sparse=True)
            sage: f = CDF(1,2) + w^5 - CDF(pi)*w + CDF(e)
            sage: f._repr()   # abs tol 1e-15
            '1.0*w^5 - 3.141592653589793*w + 3.718281828459045 + 2.0*I'
            sage: f._repr(name='z')   # abs tol 1e-15
            '1.0*z^5 - 3.141592653589793*z + 3.718281828459045 + 2.0*I'

        TESTS::

            sage: pol = RIF['x']([0, 0, (-1,1)])                                        # needs sage.rings.real_interval_field
            sage: PolynomialRing(RIF, 'x', sparse=True)(pol)                            # needs sage.rings.real_interval_field
            0.?*x^2

        AUTHOR:

        - David Harvey (2006-08-05), based on Polynomial._repr()
        - Francis Clarke (2008-09-08) improved for 'negative' coefficients
        """
        s = " "
        m = self.degree() + 1
        if name is None:
            name = self.parent().variable_name()
        atomic_repr = self.parent().base_ring()._repr_option('element_is_atomic')
        coeffs = sorted(self.__coeffs.items())
        for (n, x) in reversed(coeffs):
            if x:
                if n != m-1:
                    s += " + "
                x = y = repr(x)
                if y.find("-") == 0:
                    y = y[1:]
                if not atomic_repr and n > 0 and (y.find("+") != -1 or y.find("-") != -1):
                    x = "(%s)" % x
                if n > 1:
                    var = "*%s^%s" % (name,n)
                elif n == 1:
                    var = "*%s" % name
                else:
                    var = ""
                s += "%s%s" % (x,var)
        s = s.replace(" + -", " - ")
        s = s.replace(" 1*"," ")
        s = s.replace(" -1*", " -")
        if s == " ":
            return "0"
        return s[1:]

    def __normalize(self):
        x = self.__coeffs
        D = [n for n, z in x.items() if not z]
        for n in D:
            del x[n]

    def __getitem__(self, n):
        """
        Return the `n`-th coefficient of this polynomial.

        Negative indexes are allowed and always return 0 (so you can
        view the polynomial as embedding Laurent series).

        EXAMPLES::

            sage: # needs sage.symbolic
            sage: R.<w> = PolynomialRing(RDF, sparse=True)
            sage: e = RDF(e)
            sage: f = sum(e^n*w^n for n in range(4)); f   # abs tol 1.1e-14
            20.085536923187664*w^3 + 7.3890560989306495*w^2 + 2.718281828459045*w + 1.0
            sage: f[1]  # abs tol 5e-16
            2.718281828459045
            sage: f[5]
            0.0
            sage: f[-1]
            0.0

            sage: R.<x> = PolynomialRing(RealField(19), sparse=True)                    # needs sage.rings.real_mpfr
            sage: f = (2-3.5*x)^3; f                                                    # needs sage.rings.real_mpfr
            -42.875*x^3 + 73.500*x^2 - 42.000*x + 8.0000

        Using slices, we can truncate polynomials::

            sage: f[:2]                                                                 # needs sage.rings.real_mpfr
            -42.000*x + 8.0000

        Any other kind of slicing is an error, see :issue:`18940`::

            sage: f[1:3]                                                                # needs sage.rings.real_mpfr
            Traceback (most recent call last):
            ...
            IndexError: polynomial slicing with a start is not defined

            sage: f[1:3:2]                                                              # needs sage.rings.real_mpfr
            Traceback (most recent call last):
            ...
            IndexError: polynomial slicing with a step is not defined

        TESTS::

            sage: f["hello"]                                                            # needs sage.rings.real_mpfr
            Traceback (most recent call last):
            ...
            TypeError: list indices must be integers, not str
        """
        if isinstance(n, slice):
            start, stop, step = n.start, n.stop, n.step
            if step is not None:
                raise IndexError("polynomial slicing with a step is not defined")
            if start is not None:
                raise IndexError("polynomial slicing with a start is not defined")
            d = self.degree() + 1
            if stop is None or stop > d:
                stop = d
            v = {key: val for key, val in self.__coeffs.items()
                 if key < stop}
            return self.parent()(v)

        try:
            n = n.__index__()
        except AttributeError:
            raise TypeError("list indices must be integers, not {0}".format(type(n).__name__))
        try:
            return self.__coeffs[n]
        except KeyError:
            return self.base_ring().zero()

    def _unsafe_mutate(self, n, value):
        r"""
        Change the coefficient of `x^n` to value.

        ** NEVER USE THIS ** -- unless you really know what you are doing.

        EXAMPLES::

            sage: # needs sage.rings.real_mpfr
            sage: R.<z> = PolynomialRing(CC, sparse=True)
            sage: f = z^2 + CC.0; f
            1.00000000000000*z^2 + 1.00000000000000*I
            sage: f._unsafe_mutate(0, 10)
            sage: f
            1.00000000000000*z^2 + 10.0000000000000

        Much more nasty::

            sage: z._unsafe_mutate(1, 0)                                                # needs sage.rings.real_mpfr
            sage: z                                                                     # needs sage.rings.real_mpfr
            0
        """
        n = int(n)
        value = self.base_ring()(value)
        x = self.__coeffs
        if n < 0:
            raise IndexError("polynomial coefficient index must be nonnegative")
        if value == 0:
            if n in x:
                del x[n]
        else:
            x[n] = value

    def list(self, copy=True):
        """
        Return a new copy of the list of the underlying
        elements of ``self``.

        EXAMPLES::

            sage: R.<z> = PolynomialRing(Integers(100), sparse=True)
            sage: f = 13*z^5 + 15*z^2 + 17*z
            sage: f.list()
            [0, 17, 15, 0, 0, 13]
        """
        zero = self.base_ring().zero()
        v = [zero] * (self.degree()+1)
        for n, x in self.__coeffs.items():
            v[n] = x
        return v

    def degree(self, gen=None):
        """
        Return the degree of this sparse polynomial.

        EXAMPLES::

            sage: R.<z> = PolynomialRing(ZZ, sparse=True)
            sage: f = 13*z^50000 + 15*z^2 + 17*z
            sage: f.degree()
            50000
        """
        if not self.__coeffs:
            return -1
        return max(self.__coeffs)

    def __floordiv__(self, right):
        """
        Return the quotient upon division (no remainder).

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: R.<x> = PolynomialRing(QQbar, sparse=True)
            sage: f = (1+2*x)^3 + 3*x; f
            8*x^3 + 12*x^2 + 9*x + 1
            sage: g = f // (1+2*x); g
            4*x^2 + 4*x + 5/2
            sage: f - g * (1+2*x)
            -3/2
            sage: f.quo_rem(1+2*x)
            (4*x^2 + 4*x + 5/2, -3/2)
        """
        P = self.parent()
        if P is parent(right):
            return self._floordiv_(right)
        d = P.base_ring()(right)
        return self.map_coefficients(lambda c: c // d)

    def _add_(self, right):
        r"""
        EXAMPLES::

            sage: R.<x> = PolynomialRing(Integers(), sparse=True)
            sage: (x^100000 + 2*x^50000) + (4*x^75000 - 2*x^50000 + 3*x)
            x^100000 + 4*x^75000 + 3*x

        AUTHOR:

        - David Harvey (2006-08-05)
        """
        output = dict(self.__coeffs)

        for (index, coeff) in right.__coeffs.items():
            if index in output:
                output[index] += coeff
            else:
                output[index] = coeff

        output = self.parent()(output, check=False)
        output.__normalize()
        return output

    def _neg_(self):
        r"""
        EXAMPLES::

            sage: R.<x> = PolynomialRing(Integers(), sparse=True)
            sage: a = x^10000000; a
            x^10000000
            sage: -a
            -x^10000000
        """
        output = { }
        for (index, coeff) in self.__coeffs.items():
            output[index] = -coeff
        output = self.parent()(output, check=False)
        return output

    def _mul_(self, right):
        r"""
        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (x^100000 - x^50000) * (x^100000 + x^50000)
             x^200000 - x^100000
            sage: (x^100000 - x^50000) * R(0)
             0

        AUTHOR:
        - David Harvey (2006-08-05)
        """
        output = {}

        for (index1, coeff1) in self.__coeffs.items():
            for (index2, coeff2) in right.__coeffs.items():
                product = coeff1 * coeff2
                index = index1 + index2
                if index in output:
                    output[index] += product
                else:
                    output[index] = product

        output = self.parent()(output, check=False)
        output.__normalize()
        return output

    def _rmul_(self, left):
        r"""
        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (x^100000 - x^50000) * (x^100000 + x^50000)
            x^200000 - x^100000
            sage: 7 * (x^100000 - x^50000)   # indirect doctest
            7*x^100000 - 7*x^50000

        AUTHOR:

        - Simon King (2011-03-31)
        """
        output = {}

        for (index, coeff) in self.__coeffs.items():
            output[index] = left * coeff

        output = self.parent()(output, check=False)
        output.__normalize()
        return output

    def _lmul_(self, right):
        r"""
        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (x^100000 - x^50000) * (x^100000 + x^50000)
            x^200000 - x^100000
            sage: (x^100000 - x^50000) * 7   # indirect doctest
            7*x^100000 - 7*x^50000

        AUTHOR:

        - Simon King (2011-03-31)
        """
        output = {}

        for (index, coeff) in self.__coeffs.items():
            output[index] = coeff * right

        output = self.parent()(output, check=False)
        output.__normalize()
        return output

    def _richcmp_(self, other, op):
        """
        Compare this polynomial with other.

        Polynomials are first compared by degree, then in dictionary order
        starting with the coefficient of largest degree.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: 3*x^100 - 12 > 12*x + 5
            True
            sage: 3*x^100 - 12 > 3*x^100 - x^50 + 5
            True
            sage: 3*x^100 - 12 < 3*x^100 - x^50 + 5
            False
            sage: x^100 + x^10 - 1 < x^100 + x^10
            True
            sage: x^100 < x^100 - x^10
            False

        TESTS::

            sage: R.<x> = PolynomialRing(QQ, sparse=True)
            sage: 2*x^2^500 > x^2^500
            True

            sage: Rd = PolynomialRing(ZZ, 'x', sparse=False)
            sage: Rs = PolynomialRing(ZZ, 'x', sparse=True)
            sage: for _ in range(100):
            ....:     pd = Rd.random_element()
            ....:     qd = Rd.random_element()
            ....:     assert bool(pd < qd) == bool(Rs(pd) < Rs(qd))
        """
        d1 = self.degree()
        d2 = other.degree()

        # Special case constant polynomials
        if d1 <= 0 and d2 <= 0:
            return richcmp(self[0], other[0], op)

        # For different degrees, compare the degree
        if d1 != d2:
            return rich_to_bool_sgn(op, d1 - d2)

        degs = set(self.__coeffs) | set(other.__coeffs)
        for i in sorted(degs, reverse=True):
            x = self[i]
            y = other[i]
            res = richcmp_item(x, y, op)
            if res is not NotImplemented:
                return res
        return rich_to_bool(op, 0)

    def shift(self, n):
        r"""
        Return this polynomial multiplied by the power `x^n`.

        If `n` is negative, terms below `x^n` will be discarded. Does
        not change this polynomial.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: p = x^100000 + 2*x + 4
            sage: type(p)
            <class 'sage.rings.polynomial.polynomial_ring.PolynomialRing_integral_domain_with_category.element_class'>
            sage: p.shift(0)
             x^100000 + 2*x + 4
            sage: p.shift(-1)
             x^99999 + 2
            sage: p.shift(-100002)
             0
            sage: p.shift(2)
             x^100002 + 2*x^3 + 4*x^2

        TESTS:

        Check that :issue:`18600` is fixed::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: p = x^2^100 - 5
            sage: p.shift(10)
            x^1267650600228229401496703205386 - 5*x^10
            sage: p.shift(-10)
            x^1267650600228229401496703205366
            sage: p.shift(1.5)                                                          # needs sage.rings.real_mpfr
            Traceback (most recent call last):
            ...
            TypeError: Attempt to coerce non-integral RealNumber to Integer

        AUTHOR:
        - David Harvey (2006-08-06)
        """
        n = ZZ(n)
        if n == 0:
            return self
        if n > 0:
            output = {index+n: coeff for index, coeff in self.__coeffs.items()}
            return self.parent()(output, check=False)
        if n < 0:
            output = {index+n:coeff for index, coeff in self.__coeffs.items() if index + n >= 0}
            return self.parent()(output, check=False)

    @coerce_binop
    def quo_rem(self, other):
        """
        Return the quotient and remainder of the Euclidean division of
        ``self`` and ``other``.

        Raises :exc:`ZeroDivisionError` if ``other`` is zero.

        Raises :exc:`ArithmeticError` if ``other`` has a nonunit leading
        coefficient and this causes the Euclidean division to fail.

        EXAMPLES::

            sage: P.<x> = PolynomialRing(ZZ, sparse=True)
            sage: R.<y> = PolynomialRing(P, sparse=True)
            sage: f = R.random_element(10)
            sage: while x.divides(f.leading_coefficient()):
            ....:     f = R.random_element(10)
            sage: g = y^5 + R.random_element(4)
            sage: q, r = f.quo_rem(g)
            sage: f == q*g + r and r.degree() < g.degree()
            True
            sage: g = x*y^5
            sage: f.quo_rem(g)
            Traceback (most recent call last):
            ...
            ArithmeticError: Division non exact
            (consider coercing to polynomials over the fraction field)
            sage: g = 0
            sage: f.quo_rem(g)
            Traceback (most recent call last):
            ...
            ZeroDivisionError: Division by zero polynomial

        If the leading coefficient of ``other`` is not a unit, Euclidean division may still work::

            sage: f = -x*y^10 + 2*x*y^7 + y^3 - 2*x^2*y^2 - y
            sage: g = x*y^5
            sage: f.quo_rem(g)
            (-y^5 + 2*y^2, y^3 - 2*x^2*y^2 - y)

        Polynomials over noncommutative rings are also allowed::

            sage: # needs sage.combinat sage.modules
            sage: HH = QuaternionAlgebra(QQ, -1, -1)
            sage: P.<x> = PolynomialRing(HH, sparse=True)
            sage: f = P.random_element(5)
            sage: g = P.random_element((0, 5))
            sage: q, r = f.quo_rem(g)
            sage: f == q*g + r
            True

        TESTS::

            sage: P.<x> = PolynomialRing(ZZ, sparse=True)
            sage: f = x^10 - 4*x^6 - 5
            sage: g = 17*x^22 + x^15 - 3*x^5 + 1
            sage: q, r = g.quo_rem(f)
            sage: g == f*q + r and r.degree() < f.degree()
            True
            sage: zero = P(0)
            sage: zero.quo_rem(f)
            (0, 0)
            sage: Q.<y> = IntegerModRing(14)[]
            sage: f = y^10 - 4*y^6 - 5
            sage: g = 17*y^22 + y^15 - 3*y^5 + 1
            sage: q, r = g.quo_rem(f)
            sage: g == f*q + r and r.degree() < f.degree()
            True
            sage: f += 2*y^10  # 3 is invertible mod 14
            sage: q, r = g.quo_rem(f)
            sage: g == f*q + r and r.degree() < f.degree()
            True

        The following shows that :issue:`16649` is indeed fixed. ::

            sage: P.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (4*x).quo_rem(2*x)
            (2, 0)

        AUTHORS:

        - Bruno Grenet (2014-07-09)
        """
        if other.is_zero():
            raise ZeroDivisionError("Division by zero polynomial")
        if self.is_zero():
            return self, self

        R = self.parent()

        d = other.degree()
        if self.degree() < d:
            return R.zero(), self

        quo = R.zero()
        rem = self

        while rem.degree() >= d:
            try:
                c = R(rem.leading_coefficient() * ~other.leading_coefficient())
            except TypeError:
                raise ArithmeticError("Division non exact (consider coercing to polynomials over the fraction field)")
            e = rem.degree() - d
            quo += c*R.one().shift(e)
            # we know that the leading coefficient of rem vanishes
            # thus we avoid doing a useless computation
            rem = rem[:rem.degree()] - c*other[:d].shift(e)
        return (quo,rem)

    @coerce_binop
    def gcd(self, other, algorithm=None):
        r"""
        Return the gcd of this polynomial and ``other``.

        INPUT:

        - ``other`` -- a polynomial defined over the same ring as this
          polynomial

        ALGORITHM:

        Two algorithms are provided:

        - ``'generic'`` -- uses the generic implementation, which depends on the
          base ring being a UFD or a field
        - ``'dense'`` -- the polynomials are converted to the dense
          representation, their gcd is computed and is converted back to the
          sparse representation

        Default is ``'dense'`` for polynomials over `\ZZ` and ``'generic'`` in the
        other cases.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: p = x^6 + 7*x^5 + 8*x^4 + 6*x^3 + 2*x^2 + x + 2
            sage: q = 2*x^4 - x^3 - 2*x^2 - 4*x - 1
            sage: gcd(p, q)
            x^2 + x + 1
            sage: gcd(p, q, algorithm='dense')
            x^2 + x + 1
            sage: gcd(p, q, algorithm='generic')
            x^2 + x + 1
            sage: gcd(p, q, algorithm='foobar')
            Traceback (most recent call last):
            ...
            ValueError: Unknown algorithm 'foobar'

        TESTS:

        Check that :issue:`19676` is fixed::

            sage: S.<y> = R[]
            sage: x.gcd(y)                                                              # needs sage.libs.singular
            1
            sage: (6*x).gcd(9)                                                          # needs sage.libs.singular
            3

        Check that :issue:`36427` is fixed::

            sage: P = PolynomialRing(ZZ, "q", sparse=True)
            sage: q = P.gen()
            sage: 2*q^-100
            2/q^100
            sage: gcd(1, q^100)
            1
            sage: gcd(q^0, q^100)
            1
        """

        from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing

        if algorithm is None:
            if self.base_ring() == ZZ:
                algorithm = "dense"
            else:
                algorithm = "generic"
        if algorithm == "dense":
            S = self.parent()
            # FLINT is faster but a bug makes the conversion extremely slow,
            # so NTL is used in those cases where the conversion is too slow. Cf
            # <https://groups.google.com/d/msg/sage-devel/6qhW90dgd1k/Hoq3N7fWe4QJ>
            sd = self.degree()
            od = other.degree()
            if ((sd < 100 or len(self.__coeffs)/sd > .06)
                    and (od < 100 or len(other.__coeffs)/od > .06)):
                implementation = "FLINT"
            else:
                implementation = "NTL"
            try:
                D = PolynomialRing(S.base_ring(),'x',implementation=implementation)
            except ImportError:
                D = PolynomialRing(S.base_ring(),'x')
            g = D(self).gcd(D(other))
            return S(g)
        elif algorithm == "generic":
            return Polynomial.gcd(self,other)
        else:
            raise ValueError("Unknown algorithm '%s'" % algorithm)

    def reverse(self, degree=None):
        """
        Return this polynomial but with the coefficients reversed.

        If an optional degree argument is given, the coefficient list will be
        truncated or zero padded as necessary and the reverse polynomial will
        have the specified degree.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: p = x^4 + 2*x^2^100
            sage: p.reverse()
            x^1267650600228229401496703205372 + 2
            sage: p.reverse(10)
            x^6
        """
        if degree is None:
            degree = self.degree()
        if not isinstance(degree, (int,Integer)):
            raise ValueError("degree argument must be a nonnegative integer, got %s" % degree)
        d = {degree-k: v for k,v in self.__coeffs.items() if degree >= k}
        return self.parent()(d, check=False)

    def truncate(self, n):
        """
        Return the polynomial of degree `< n` equal to ``self`` modulo `x^n`.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: (x^11 + x^10 + 1).truncate(11)
            x^10 + 1
            sage: (x^2^500 + x^2^100 + 1).truncate(2^101)
            x^1267650600228229401496703205376 + 1
        """
        return self[:n]

    def number_of_terms(self):
        """
        Return the number of nonzero terms.

        EXAMPLES::

            sage: R.<x> = PolynomialRing(ZZ, sparse=True)
            sage: p = x^100 - 3*x^10 + 12
            sage: p.number_of_terms()
            3
        """
        return len(self.__coeffs)


class Polynomial_generic_domain(Polynomial, IntegralDomainElement):
    def __init__(self, parent, is_gen=False, construct=False):
        Polynomial.__init__(self, parent, is_gen=is_gen)

    def is_unit(self):
        r"""
        Return ``True`` if this polynomial is a unit.

        *EXERCISE* (Atiyah-McDonald, Ch 1): Let `A[x]` be a polynomial
        ring in one variable.  Then `f=\sum a_i x^i \in A[x]` is a
        unit if and only if `a_0` is a unit and `a_1,\ldots, a_n` are
        nilpotent.

        EXAMPLES::

            sage: R.<z> = PolynomialRing(ZZ, sparse=True)
            sage: (2 + z^3).is_unit()
            False
            sage: f = -1 + 3*z^3; f
            3*z^3 - 1
            sage: f.is_unit()
            False
            sage: R(-3).is_unit()
            False
            sage: R(-1).is_unit()
            True
            sage: R(0).is_unit()
            False
        """
        if self.degree() > 0:
            return False
        return self[0].is_unit()


class Polynomial_generic_field(Polynomial_singular_repr,
                               Polynomial_generic_domain,
                               EuclideanDomainElement):

    @coerce_binop
    def quo_rem(self, other):
        """
        Return a tuple ``(quotient, remainder)`` where
        ``self = quotient * other + remainder``.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: R.<y> = PolynomialRing(QQ)
            sage: K.<t> = NumberField(y^2 - 2)
            sage: P.<x> = PolynomialRing(K)
            sage: x.quo_rem(K(1))
            (x, 0)
            sage: x.xgcd(K(1))
            (1, 0, 1)
        """
        P = self.parent()
        if other.is_zero():
            raise ZeroDivisionError("other must be nonzero")

        # This is algorithm 3.1.1 in Cohen GTM 138
        A = self
        B = other
        R = A
        Q = P.zero()
        while R.degree() >= B.degree():
            aaa = R.leading_coefficient()/B.leading_coefficient()
            diff_deg = R.degree()-B.degree()
            Q += P(aaa).shift(diff_deg)
            # We know that S*B exactly cancels the leading coefficient of R.
            # Thus, we skip the computation of this leading coefficient.
            # For most exact fields, this doesn't matter much; but for
            # inexact fields, the leading coefficient might not end up
            # exactly equal to zero; and for AA/QQbar, verifying that
            # the coefficient is exactly zero triggers exact computation.
            R = R[:R.degree()] - (aaa*B[:B.degree()]).shift(diff_deg)
        return (Q, R)


class Polynomial_generic_sparse_field(Polynomial_generic_sparse, Polynomial_generic_field):
    """
    EXAMPLES::

        sage: R.<x> = PolynomialRing(Frac(RR['t']), sparse=True)
        sage: f = x^3 - x + 17
        sage: type(f)
        <class 'sage.rings.polynomial.polynomial_ring.PolynomialRing_field_with_category.element_class'>
        sage: loads(f.dumps()) == f
        True
    """
    def __init__(self, parent, x=None, check=True, is_gen=False, construct=False):
        Polynomial_generic_sparse.__init__(self, parent, x, check, is_gen)


class Polynomial_generic_dense_field(Polynomial_generic_dense, Polynomial_generic_field):
    def __init__(self, parent, x=None, check=True, is_gen=False, construct=False):
        Polynomial_generic_dense.__init__(self, parent, x, check, is_gen)


##########################################
# Over discrete valuation rings and fields
##########################################

class Polynomial_generic_cdv(Polynomial_generic_domain):
    """
    A generic class for polynomials over complete discrete
    valuation domains and fields.

    AUTHOR:

    - Xavier Caruso (2013-03)
    """
    def newton_slopes(self, repetition=True):
        """
        Return a list of the Newton slopes of this polynomial.

        These are the valuations of the roots of this polynomial.

        If ``repetition`` is ``True``, each slope is repeated a number of
        times equal to its multiplicity. Otherwise it appears only
        one time.

        EXAMPLES::

            sage: # needs sage.geometry.polyhedron sage.rings.padics
            sage: K = Qp(5)
            sage: R.<t> = K[]
            sage: f = 5 + 3*t + t^4 + 25*t^10
            sage: f.newton_polygon()
            Finite Newton polygon with 4 vertices: (0, 1), (1, 0), (4, 0), (10, 2)
            sage: f.newton_slopes()
            [1, 0, 0, 0, -1/3, -1/3, -1/3, -1/3, -1/3, -1/3]
            sage: f.newton_slopes(repetition=False)
            [1, 0, -1/3]

        AUTHOR:

        - Xavier Caruso (2013-03-20)
        """
        polygon = self.newton_polygon()
        return [-s for s in polygon.slopes(repetition=repetition)]

    def newton_polygon(self):
        r"""
        Return a list of vertices of the Newton polygon of this polynomial.

        .. NOTE::

            If some coefficients have not enough precision an error is raised.

        EXAMPLES::

            sage: # needs sage.geometry.polyhedron sage.rings.padics
            sage: K = Qp(5)
            sage: R.<t> = K[]
            sage: f = 5 + 3*t + t^4 + 25*t^10
            sage: f.newton_polygon()
            Finite Newton polygon with 4 vertices: (0, 1), (1, 0), (4, 0), (10, 2)
            sage: g = f + K(0,0)*t^4; g
            (5^2 + O(5^22))*t^10 + O(5^0)*t^4 + (3 + O(5^20))*t + 5 + O(5^21)
            sage: g.newton_polygon()
            Traceback (most recent call last):
            ...
            PrecisionError: The coefficient of t^4 has not enough precision

        TESTS:

        Check that :issue:`22936` is fixed::

            sage: S.<x> = PowerSeriesRing(GF(5))
            sage: R.<y> = S[]
            sage: p = x^2 + y + x*y^2
            sage: p.newton_polygon()                                                    # needs sage.geometry.polyhedron
            Finite Newton polygon with 3 vertices: (0, 2), (1, 0), (2, 1)

        AUTHOR:

        - Xavier Caruso (2013-03-20)
        """
        d = self.degree()
        from sage.geometry.newton_polygon import NewtonPolygon
        polygon = NewtonPolygon([(x, self[x].valuation()) for x in range(d+1)])
        polygon_prec = NewtonPolygon([ (x, self[x].precision_absolute()) for x in range(d+1) ])
        vertices = polygon.vertices(copy=False)
        vertices_prec = polygon_prec.vertices(copy=False)
        if len(vertices_prec) > 0:
            if vertices[0][0] > vertices_prec[0][0]:
                raise PrecisionError("first term with non-infinite valuation must have determined valuation")
            elif vertices[-1][0] < vertices_prec[-1][0]:
                raise PrecisionError("last term with non-infinite valuation must have determined valuation")
            else:
                for (x, y) in vertices:
                    if polygon_prec(x) <= y:
                        raise PrecisionError("The coefficient of %s^%s has not enough precision" % (self.parent().variable_name(), x))
        return polygon

    def hensel_lift(self, a):
        """
        Lift `a` to a root of this polynomial (using
        Newton iteration).

        If `a` is not close enough to a root (so that
        Newton iteration does not converge), an error
        is raised.

        EXAMPLES::

            sage: # needs sage.rings.padics
            sage: K = Qp(5, 10)
            sage: P.<x> = PolynomialRing(K)
            sage: f = x^2 + 1
            sage: root = f.hensel_lift(2); root
            2 + 5 + 2*5^2 + 5^3 + 3*5^4 + 4*5^5 + 2*5^6 + 3*5^7 + 3*5^9 + O(5^10)
            sage: f(root)
            O(5^10)

            sage: g = (x^2 + 1) * (x - 7)                                               # needs sage.rings.padics
            sage: g.hensel_lift(2)  # here, 2 is a multiple root modulo p               # needs sage.rings.padics
            Traceback (most recent call last):
            ...
            ValueError: a is not close enough to a root of this polynomial

        AUTHOR:

        - Xavier Caruso (2013-03-23)
        """
        selfa = self(a)
        der = self.derivative()
        dera = der(a)
        if selfa.valuation() <= 2 * dera.valuation():
            raise ValueError("a is not close enough to a root of this polynomial")
        # Newton iteration
        # Todo: compute everything up to the adequate precision at each step
        b = ~dera
        while True:
            na = a - selfa * b
            if na == a:
                return a
            a = na
            selfa = self(a)
            dera = der(a)
            b *= 2 - dera * b

    def _factor_of_degree(self, deg):
        """
        Return a factor of ``self`` of degree ``deg``.

        Algorithm is Newton iteration.

        This fails if ``deg`` is not a breakpoint in the Newton
        polygon of ``self``.

        Only for internal use!

        EXAMPLES::

            sage: # needs sage.rings.padics
            sage: K = Qp(5)
            sage: R.<x> = K[]
            sage: K = Qp(5)
            sage: R.<t> = K[]
            sage: f = 5 + 3*t + t^4 + 25*t^10

            sage: g = f._factor_of_degree(4)                                            # needs sage.rings.padics
            sage: (f % g).is_zero()                                                     # needs sage.rings.padics
            True

            sage: g = f._factor_of_degree(3)    # not tested                            # needs sage.rings.padics
            Traceback (most recent call last)
            ...
            KeyboardInterrupt:

        TESTS::

            sage: S.<x> = PowerSeriesRing(GF(5))
            sage: R.<y> = S[]
            sage: p = x^2 + y + x*y^2
            sage: p._factor_of_degree(1)
            (1 + O(x^20))*y + x^2 + x^5 + 2*x^8 + 4*x^14 + 2*x^17 + 2*x^20 + O(x^22)

        AUTHOR:

        - Xavier Caruso (2013-03-20)

        .. TODO::

            Precision is not optimal, and can be improved.
        """
        coeffs = self.list()
        a = coeffs[:deg+1]
        # The leading coefficient need to be known at finite precision
        # in order to ensure that the while loop below terminates
        if a[deg].precision_absolute() is Infinity:
            a[deg] = a[deg].add_bigoh(self.base_ring().default_prec())

        parent = self.parent()
        a = parent(a)
        v = parent.one()
        x = self % a
        while not x.is_zero():
            a += (v * x) % a
            b, x = self.quo_rem(a)
            b %= a
            v = (v * (2 - b * v)) % a
        return a

    def factor_of_slope(self, slope=None):
        """
        INPUT:

        - ``slope`` -- a rational number (default: the first slope
          in the Newton polygon of ``self``)

        OUTPUT:

        The factor of ``self`` corresponding to the slope ``slope`` (i.e.
        the unique monic divisor of ``self`` whose slope is ``slope`` and
        degree is the length of ``slope`` in the Newton polygon).

        EXAMPLES::

            sage: # needs sage.geometry.polyhedron sage.rings.padics
            sage: K = Qp(5)
            sage: R.<x> = K[]
            sage: K = Qp(5)
            sage: R.<t> = K[]
            sage: f = 5 + 3*t + t^4 + 25*t^10
            sage: f.newton_slopes()
            [1, 0, 0, 0, -1/3, -1/3, -1/3, -1/3, -1/3, -1/3]
            sage: g = f.factor_of_slope(0)
            sage: g.newton_slopes()
            [0, 0, 0]
            sage: (f % g).is_zero()
            True
            sage: h = f.factor_of_slope()
            sage: h.newton_slopes()
            [1]
            sage: (f % h).is_zero()
            True

        If ``slope`` is not a slope of ``self``, the corresponding factor
        is `1`::

            sage: f.factor_of_slope(-1)                                                 # needs sage.geometry.polyhedron sage.rings.padics
            1 + O(5^20)

        AUTHOR:

        - Xavier Caruso (2013-03-20)
        """
        one = self.parent()(1)
        vertices = self.newton_polygon().vertices(copy=False)
        if len(vertices) < 2:
            if slope is Infinity:
                return self.parent().gen() ** self.degree()
            else:
                return one
        if slope is None:
            deg_first = vertices[0][0]
            deg_last = vertices[1][0]
        else:
            (deg_first, y_first) = vertices[0]
            for i in range(1, len(vertices)):
                (deg_last, y_last) = vertices[i]
                slope_cur = (y_first - y_last) / (deg_last - deg_first)
                if slope_cur == slope:
                    break
                elif slope_cur < slope:
                    return one
                deg_first = deg_last
                y_first = y_last
            if slope_cur > slope:
                return one
        if deg_last == self.degree():
            div = self
        else:
            div = self._factor_of_degree(deg_last)
        if deg_first > 0:
            div2 = div._factor_of_degree(deg_first)
            div,_ = div.quo_rem(div2)
        return div.monic()

    def slope_factorization(self):
        """
        Return a factorization of ``self`` into a product of factors
        corresponding to each slope in the Newton polygon.

        EXAMPLES::

            sage: # needs sage.geometry.polyhedron sage.rings.padics
            sage: K = Qp(5)
            sage: R.<x> = K[]
            sage: K = Qp(5)
            sage: R.<t> = K[]
            sage: f = 5 + 3*t + t^4 + 25*t^10
            sage: f.newton_slopes()
            [1, 0, 0, 0, -1/3, -1/3, -1/3, -1/3, -1/3, -1/3]
            sage: F = f.slope_factorization()
            sage: F.prod() == f
            True
            sage: for (f,_) in F:
            ....:     print(f.newton_slopes())
            [-1/3, -1/3, -1/3, -1/3, -1/3, -1/3]
            [0, 0, 0]
            [1]

        TESTS::

            sage: S.<x> = PowerSeriesRing(GF(5))
            sage: R.<y> = S[]
            sage: p = x^2 + y + x*y^2
            sage: p.slope_factorization()                                               # needs sage.geometry.polyhedron
            (x)
            * ((x + O(x^22))*y + 1 + 4*x^3 + 4*x^6 + 3*x^9 + x^15 + 3*x^18 + O(x^21))
            * ((x^-1 + O(x^20))*y + x + x^4 + 2*x^7 + 4*x^13 + 2*x^16 + 2*x^19 + O(x^22))

        AUTHOR:

        - Xavier Caruso (2013-03-20)
        """
        vertices = self.newton_polygon().vertices(copy=False)

        unit = self.leading_coefficient()
        P = ~unit * self

        deg_first = vertices[0][0]
        factors = [ ]
        if deg_first > 0:
            P >>= deg_first
            factors.append((self._parent.gen(), deg_first))
        if len(vertices) > 2:
            for i in range(1, len(vertices)-1):
                deg = vertices[i][0]
                div = P._factor_of_degree(deg-deg_first)
                factors.append((div,1))
                P,_ = P.quo_rem(div)
                deg_first = deg
        if len(vertices) > 1:
            factors.append((P, 1))
        factors.reverse()
        return Factorization(factors, sort=False, unit=unit)

    def _roots(self, secure, minval, hint):
        """
        Return the roots of this polynomial whose valuation is
        at least ``minval``.

        This is a helper method for :meth:`roots`.
        It is not meant to be called directly.

        INPUT:

        - ``secure`` -- boolean; whether we raise an error or
          not in case of multiple roots

        - ``minval`` -- integer

        - ``hint`` -- list or ``None``; if given, it must be the
          list of roots of the residual polynomial of slope ``minval``

        OUTPUT: list of pairs ``(root, multiplicity)``

        TESTS::

            sage: # needs sage.geometry.polyhedron sage.rings.padics
            sage: R = Zp(2)
            sage: S.<x> = R[]
            sage: P = (x-1) * (x-2) * (x-4) * (x-8) * (x-16)
            sage: Q = P^2
            sage: Q.roots(algorithm='sage')  # indirect doctest
            [(2^4 + O(2^14), 2),
             (2^3 + O(2^13), 2),
             (2^2 + O(2^12), 2),
             (2 + O(2^11), 2),
             (1 + O(2^10), 2)]
        """
        from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
        K = self.base_ring()
        Pk = PolynomialRing(K.residue_field(), names='xbar')
        x = self.parent().gen()

        # Trivial cases
        if self.degree() == 0:
            return [ ]
        if self.degree() == 1:
            return [ (-self[0]/self[1], 1) ]

        # We consider the case where zero is a (possibly multiple) root
        i = 0
        while self[i] == 0:
            i += 1
        if secure and i > 1:
            raise PrecisionError("not enough precision to determine the number of roots")
        if i == 0:
            roots = [ ]
            P = self
        else:
            vali = self[i].valuation()
            prec = min((self[j].precision_absolute()-vali) / (i-j) for j in range(i))
            if prec is not Infinity:
                prec = prec.ceil()
            roots = [ (K(0,prec), i) ]
            P = self // self[:i+1]  # we do not shift because we need to track precision here

        # We use Newton polygon and slope factorisation to find roots
        vertices = P.newton_polygon().vertices(copy=False)
        deg = 0
        for i in range(1, len(vertices)):
            deg_left, val_left = vertices[i-1]
            deg_right, val_right = vertices[i]
            slope = (val_right - val_left) / (deg_left - deg_right)
            if slope not in ZZ or slope < minval:
                continue
            if hint is not None and slope == minval:
                rootsbar = hint
                if not rootsbar:
                    continue
            if i < len(vertices) - 1:
                F = P._factor_of_degree(deg_right - deg)
                P = P // F
            else:
                F = P
            if deg < deg_left:
                G = F._factor_of_degree(deg_left - deg)
                F //= G
            deg = deg_right
            val = F[0].valuation()
            if hint is None or slope != minval:
                Fbar = Pk([ F[j] >> (val - j*slope) for j in range(F.degree()+1) ])
                rootsbar = [ r for (r, _) in Fbar.roots() ]
                if not rootsbar:
                    continue
            rbar = rootsbar.pop()
            shift = K(rbar).lift_to_precision() << slope  # probably we should choose a better lift
            roots += [(r+shift, m) for (r, m) in F(x+shift)._roots(secure, slope, [r-rbar for r in rootsbar])]  # recursive call
        return roots


class Polynomial_generic_dense_cdv(Polynomial_generic_dense_inexact, Polynomial_generic_cdv):
    pass


class Polynomial_generic_sparse_cdv(Polynomial_generic_sparse, Polynomial_generic_cdv):
    pass


class Polynomial_generic_cdvr(Polynomial_generic_cdv):
    pass


class Polynomial_generic_dense_cdvr(Polynomial_generic_dense_cdv, Polynomial_generic_cdvr):
    pass


class Polynomial_generic_sparse_cdvr(Polynomial_generic_sparse_cdv, Polynomial_generic_cdvr):
    pass


class Polynomial_generic_cdvf(Polynomial_generic_cdv, Polynomial_generic_field):
    pass


class Polynomial_generic_dense_cdvf(Polynomial_generic_dense_cdv, Polynomial_generic_cdvf):
    pass


class Polynomial_generic_sparse_cdvf(Polynomial_generic_sparse_cdv, Polynomial_generic_cdvf):
    pass


############################################################################
# XXX:  Ensures that the generic polynomials implemented in Sage via PARI  #
# until at least until 4.5.0 unpickle correctly as polynomials implemented #
# via FLINT.                                                               #
try:
    from sage.rings.polynomial.polynomial_rational_flint import Polynomial_rational_flint
except ImportError:
    pass
else:
    from sage.misc.persist import register_unpickle_override
    register_unpickle_override('sage.rings.polynomial.polynomial_element_generic',
                               'Polynomial_rational_dense', Polynomial_rational_flint)



# sage_setup: distribution = sagemath-categories
cimport cython

cdef struct pair_s:
    size_t first
    size_t second


@cython.final
cdef class ListOfPairs:
    cdef pair_s** _lists
    cdef size_t length

    cdef inline int enlarge(self) except -1
    cdef inline int add(self, size_t first, size_t second) except -1
    cdef inline pair_s* get(self, size_t index) except NULL



# sage_setup: distribution = sagemath-categories
cdef class CommutativePolynomial(CommutativeAlgebraElement):
    r"""
    Abstract base class for commutative polynomials in any number of variables.

    It is a common base for :class:`~sage.rings.polynomial.polynomial_element.Polynomial`,
    :class:`~sage.rings.polynomial.multi_polynomial.MPolynomial`, and
    :class:`~sage.rings.polynomial.infinite_polynomial_element.InfinitePolynomial`.

    EXAMPLES::

        sage: from sage.rings.polynomial.commutative_polynomial import CommutativePolynomial
        sage: K.<x> = PolynomialRing(QQ)
        sage: isinstance(x, CommutativePolynomial)
        True
        sage: K.<x,y> = PolynomialRing(QQ)
        sage: isinstance(x, CommutativePolynomial)
        True
        sage: X.<x,y> = InfinitePolynomialRing(ZZ, implementation='sparse')
        sage: isinstance(x[2], CommutativePolynomial)
        True
    """

    pass



# sage_setup: distribution = sagemath-categories

from sage.misc.lazy_import import lazy_import

# Ring base classes
from sage.rings.ring import (Ring, Field, CommutativeRing, IntegralDomain,
                             PrincipalIdealDomain)

lazy_import("sage.rings.ring", "DedekindDomain")

# Ring element base classes
from sage.structure.element import (CommutativeAlgebraElement,
                                    RingElement, CommutativeRingElement, IntegralDomainElement,
                                    DedekindDomainElement, PrincipalIdealDomainElement,
                                    EuclideanDomainElement, FieldElement)

# Integers modulo n.
from sage.rings.finite_rings.integer_mod_ring import IntegerModRing, Zmod
from sage.rings.finite_rings.integer_mod import IntegerMod, Mod, mod
Integers = IntegerModRing

# Quotient
from sage.rings.quotient_ring import QuotientRing

# Infinities
from sage.rings.infinity import infinity, Infinity, InfinityRing, unsigned_infinity, UnsignedInfinityRing
oo = infinity

# Rational integers.
from sage.rings.integer_ring import IntegerRing, ZZ, crt_basis
from sage.rings.integer import Integer

# Rational numbers
from sage.rings.rational import Rational
from sage.rings.rational_field import RationalField, QQ
Rationals = RationalField

# Integers modulo n.
from sage.rings.finite_rings.integer_mod_ring import IntegerModRing, Zmod
from sage.rings.finite_rings.integer_mod import IntegerMod, Mod, mod
Integers = IntegerModRing

# Function field
from sage.rings.function_field.all__sagemath_categories import *

# Double precision floating point numbers
from sage.rings.real_double import RealDoubleField, RDF, RealDoubleElement

# Lazy reals
from sage.rings.real_lazy import RealLazyField, RLF, ComplexLazyField, CLF

# Ideals
from sage.rings.ideal import Ideal
ideal = Ideal

# Semirings
from sage.rings.semirings.all import *

from sage.rings.finite_rings.all__sagemath_categories import *
from sage.rings.function_field.all__sagemath_categories import *
from sage.rings.number_field.all__sagemath_categories import *
from sage.rings.padics.all__sagemath_categories import *
from sage.rings.polynomial.all__sagemath_categories import *

# Power series rings
from sage.rings.power_series_ring import PowerSeriesRing

# Laurent series ring in one variable
from sage.rings.laurent_series_ring import LaurentSeriesRing

# Puiseux series ring
from sage.rings.puiseux_series_ring import PuiseuxSeriesRing

# Big-oh notation
from sage.rings.big_oh import O

# Fraction field
from sage.rings.fraction_field import FractionField
Frac = FractionField

# Localization
from sage.rings.localization import Localization

# continued fractions
from sage.rings.continued_fraction import (continued_fraction,
                                           continued_fraction_list)

# Preliminary version of real numbers for doctesting without sage.rings.real_mpfr.
# sage.rings.all redefines it.
RealNumber = RR = RDF             # used by the preparser to wrap real literals

del lazy_import



# sage_setup: distribution = sagemath-categories
"Polynomial multiplication by Kronecker substitution"
################################################################################
#       Copyright (C) 2007 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
################################################################################
from sage.rings.rational_field import QQ
from sage.rings.integer_ring import ZZ

# Faster than Sage's
from math import log as pylog
from math import ceil as pyceil


def _mul_fateman_to_int2(f_list, g_list):
    """
    Convert a polynomial to an integer by evaluating it.

    INPUT:

    - ``f_list``, ``g_list`` -- lists of integers

    OUTPUT: padding
    """
    max_coeff_f = max([abs(i) for i in f_list])
    max_coeff_g = max([abs(i) for i in g_list])
    b = (1 + min(len(f_list), len(g_list))) * max_coeff_f * max_coeff_g
    return int(pyceil(pylog(b, 2)))


def _mul_fateman_to_poly(number, padding):
    """
    Convert a number to a polynomial, according to a padding.

    OUTPUT: list containing the coefficient of a polynomial of degree
    ``len(list)``
    """
    coeffs = []
    flag = 0
    append = coeffs.append
    if number < 0:
        number = -number
        flag = 1

    while number > 0:
        r = number % (1 << padding)
        number = (number - r) >> padding
        if r > (1 << (padding - 1)):
            r -= 1 << padding
            number += 1
        append(r)

    if flag == 1:
        return [-c for c in coeffs]
    return coeffs


def _mul_fateman_mul(f, g):
    """
    Multiply 2 polynomials
    """
    f = f.change_ring(QQ)
    g = g.change_ring(QQ)

    f_list = f.list()
    g_list = g.list()

    # If these polynomials have real
    # coefficients, convert them to
    # rational coefficients.
    # Note: no precision is lost in this
    # direction

    fgcd = f_list[0].content(f_list)
    ggcd = g_list[0].content(g_list)

    # Need to change ring to ZZ
    z_poly_f = (f * fgcd.denominator()).change_ring(ZZ)
    z_poly_g = (g * ggcd.denominator()).change_ring(ZZ)

    div = 1 / (fgcd.denominator() * ggcd.denominator())

    z_poly_f_list = z_poly_f.coefficients(sparse=False)
    z_poly_g_list = z_poly_g.coefficients(sparse=False)
    padding = _mul_fateman_to_int2(z_poly_f_list, z_poly_g_list)

    n_f = z_poly_f(1 << padding)
    n_g = z_poly_g(1 << padding)

    if div == 1:
        return _mul_fateman_to_poly(n_f * n_g, padding)
    else:
        l = _mul_fateman_to_poly(n_f * n_g, padding)
        return [QQ(i * div) for i in l]
